@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: app.c 11402 2010-09-21 08:07:06Z yeti-dn $
+ *  @(#) $Id: app.c 12238 2011-03-25 22:04:24Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -27,6 +27,7 @@
 #include <libgwyddion/gwymacros.h>
 #include <libprocess/datafield.h>
 #include <libprocess/gwygrainvalue.h>
+#include <libprocess/gwycalibration.h>
 #include <libgwymodule/gwymodule.h>
 #include <libgwydgets/gwydgets.h>
 #include <libgwydgets/gwygraphwindow.h>
@@ -43,28 +44,33 @@ static GtkWidget *gwy_app_main_window = NULL;
 static GwyTool* current_tool = NULL;
 static GQuark corner_item_quark = 0;
 
-static gboolean   gwy_app_main_window_save_position(void);
+static gboolean   gwy_app_main_window_save_position   (void);
 static void       gwy_app_main_window_restore_position(void);
-static gboolean   gwy_app_confirm_quit             (void);
-static void       gather_unsaved_cb                (GwyDataWindow *data_window,
-                                                    GSList **unsaved);
-static gboolean   gwy_app_confirm_quit_dialog      (GSList *unsaved);
-static GtkWidget* gwy_app_menu_data_popup_create   (GtkAccelGroup *accel_group);
-static GtkWidget* gwy_app_menu_data_corner_create  (GtkAccelGroup *accel_group);
-static void       gwy_app_data_window_change_square(GtkWidget *item,
-                                                    gpointer user_data);
+static gboolean   gwy_app_confirm_quit                (void);
+static void       gather_unsaved_cb                   (GwyDataWindow *data_window,
+                                                       GSList **unsaved);
+static gboolean   gwy_app_confirm_quit_dialog         (GSList *unsaved);
+static GtkWidget* gwy_app_menu_data_popup_create      (GtkAccelGroup *accel_group);
+static GtkWidget* gwy_app_menu_data_corner_create     (GtkAccelGroup *accel_group);
+static void       gwy_app_data_window_change_square   (GtkWidget *item,
+                                                       gpointer user_data);
 static gboolean   gwy_app_data_corner_menu_popup_mouse(GtkWidget *menu,
                                                        GdkEventButton *event,
                                                        GtkWidget *ebox);
-static gboolean   gwy_app_data_popup_menu_popup_mouse(GtkWidget *menu,
-                                                      GdkEventButton *event,
-                                                      GwyDataView *data_view);
-static void       gwy_app_data_popup_menu_popup_key(GtkWidget *menu,
-                                                    GtkWidget *data_window);
-static void       gwy_app_3d_window_export         (Gwy3DWindow *window);
-static void       gwy_app_3d_view_set_defaults     (Gwy3DView *gwy3dview);
-static void       gwy_app_data_window_reset_zoom   (void);
-static void       gwy_app_change_mask_color_cb     (void);
+static gboolean   gwy_app_data_popup_menu_popup_mouse (GtkWidget *menu,
+                                                       GdkEventButton *event,
+                                                       GwyDataView *data_view);
+static void       gwy_app_data_popup_menu_popup_key   (GtkWidget *menu,
+                                                       GtkWidget *data_window);
+static gboolean   gwy_app_graph_popup_menu_popup_mouse(GtkWidget *menu,
+                                                       GdkEventButton *event,
+                                                       GwyGraph *graph);
+static void       gwy_app_graph_popup_menu_popup_key  (GtkWidget *menu,
+                                                       GtkWidget *graph);
+static void       gwy_app_3d_window_export            (Gwy3DWindow *window);
+static void       gwy_app_3d_view_set_defaults        (Gwy3DView *gwy3dview);
+static void       gwy_app_data_window_reset_zoom      (void);
+static void       gwy_app_change_mask_color_cb        (void);
 
 /*****************************************************************************
  *                                                                           *
@@ -362,6 +368,287 @@ _gwy_app_data_window_setup(GwyDataWindow *data_window)
     }
 }
 
+static GtkWidget*
+gwy_app_menu_data_popup_create(GtkAccelGroup *accel_group)
+{
+    static struct {
+        const gchar *label;
+        gpointer callback;
+        gpointer cbdata;
+        guint key;
+        GdkModifierType mods;
+    }
+    const menu_items[] = {
+        {
+            NULL, gwy_app_run_process_func,
+            "mask_remove", GDK_K, GDK_CONTROL_MASK
+        },
+        {
+            N_("Mask _Color..."),  gwy_app_change_mask_color_cb,
+            NULL, 0, 0
+        },
+        {
+            NULL, gwy_app_run_process_func,
+            "fix_zero", 0, 0
+        },
+        {
+            NULL, gwy_app_run_process_func,
+            "presentation_remove", GDK_K, GDK_CONTROL_MASK | GDK_SHIFT_MASK
+        },
+        {
+            NULL, gwy_app_run_process_func,
+            "level", GDK_L, GDK_CONTROL_MASK
+        },
+        {
+            N_("Zoom _1:1"), gwy_app_data_window_reset_zoom,
+            NULL, 0, 0
+        },
+    };
+    GwySensitivityGroup *sensgroup;
+    GtkWidget *menu, *item;
+    const gchar *name;
+    guint i, mask;
+
+    menu = gtk_menu_new();
+    if (accel_group)
+        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);
+    sensgroup = gwy_app_sensitivity_get_group();
+    for (i = 0; i < G_N_ELEMENTS(menu_items); i++) {
+        if (menu_items[i].callback == gwy_app_run_process_func
+            && !gwy_process_func_get_run_types((gchar*)menu_items[i].cbdata)) {
+            g_warning("Processing function <%s> for "
+                      "data view context menu is not available.",
+                      (const gchar*)menu_items[i].cbdata);
+            continue;
+        }
+        if (menu_items[i].callback == gwy_app_run_process_func) {
+            name = _(gwy_process_func_get_menu_path(menu_items[i].cbdata));
+            name = strrchr(name, '/');
+            if (!name) {
+                g_warning("Invalid translated menu path for <%s>",
+                          (const gchar*)menu_items[i].cbdata);
+                continue;
+            }
+            item = gtk_menu_item_new_with_mnemonic(name + 1);
+            mask = gwy_process_func_get_sensitivity_mask(menu_items[i].cbdata);
+            gwy_sensitivity_group_add_widget(sensgroup, item, mask);
+        }
+        else {
+            item = gtk_menu_item_new_with_mnemonic(_(menu_items[i].label));
+
+            if (menu_items[i].callback == gwy_app_change_mask_color_cb)
+                gwy_sensitivity_group_add_widget(sensgroup, item,
+                                                 GWY_MENU_FLAG_DATA_MASK);
+        }
+
+        if (menu_items[i].key)
+            gtk_widget_add_accelerator(item, "activate", accel_group,
+                                       menu_items[i].key, menu_items[i].mods,
+                                       GTK_ACCEL_VISIBLE | GTK_ACCEL_LOCKED);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        g_signal_connect_swapped(item, "activate",
+                                 G_CALLBACK(menu_items[i].callback),
+                                 menu_items[i].cbdata);
+    }
+
+    return menu;
+}
+
+static gboolean
+gwy_app_data_popup_menu_popup_mouse(GtkWidget *menu,
+                                    GdkEventButton *event,
+                                    GwyDataView *data_view)
+{
+    if (event->button != 3)
+        return FALSE;
+
+    gwy_app_data_browser_select_data_view(data_view);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                   event->button, event->time);
+
+    return TRUE;
+}
+
+static void
+gwy_app_data_popup_menu_position(G_GNUC_UNUSED GtkMenu *menu,
+                                 gint *x,
+                                 gint *y,
+                                 gboolean *push_in,
+                                 GtkWidget *window)
+{
+    GwyDataView *data_view;
+
+    data_view = gwy_data_window_get_data_view(GWY_DATA_WINDOW(window));
+    gdk_window_get_origin(GTK_WIDGET(data_view)->window, x, y);
+    *push_in = TRUE;
+}
+
+static void
+gwy_app_data_popup_menu_popup_key(GtkWidget *menu,
+                                  GtkWidget *data_window)
+{
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+                   (GtkMenuPositionFunc)gwy_app_data_popup_menu_position,
+                   data_window,
+                   0, gtk_get_current_event_time());
+}
+
+/**
+ * gwy_app_data_corner_menu_update:
+ * @menu: Data window corner menu.
+ * @data_view: The corresponding data view.
+ *
+ * Updates corner menu to reflect data window's state before we show it.
+ **/
+static void
+gwy_app_data_corner_menu_update(GtkWidget *menu,
+                                GwyDataView *data_view)
+{
+    gboolean realsquare = FALSE;
+    GwyContainer *data;
+    const gchar *key;
+    gchar *s;
+    GtkWidget *item;
+    GList *l;
+    gulong id;
+    guint i;
+
+    /* Square mode */
+    data = gwy_data_view_get_data(data_view);
+    key = gwy_data_view_get_data_prefix(data_view);
+    s = g_strconcat(key, "/realsquare", NULL);
+    gwy_container_gis_boolean_by_name(data, s, &realsquare);
+    gwy_debug("view's realsquare: %d", realsquare);
+    g_free(s);
+
+    /* Update stuff */
+    l = gtk_container_get_children(GTK_CONTAINER(menu));
+    while (l) {
+        item = GTK_WIDGET(l->data);
+        i = GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(item),
+                                                corner_item_quark));
+        switch (i) {
+            case ITEM_PIXELSQUARE:
+            if (!realsquare) {
+                gwy_debug("setting Pixelwise active");
+                id = g_signal_handler_find(item, G_SIGNAL_MATCH_FUNC,
+                                           0, 0, NULL,
+                                           gwy_app_data_window_change_square,
+                                           NULL);
+                g_signal_handler_block(item, id);
+                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+                g_signal_handler_unblock(item, id);
+            }
+            break;
+
+            case ITEM_REALSQUARE:
+            if (realsquare) {
+                gwy_debug("setting Physical active");
+                id = g_signal_handler_find(item, G_SIGNAL_MATCH_FUNC,
+                                           0, 0, NULL,
+                                           gwy_app_data_window_change_square,
+                                           NULL);
+                g_signal_handler_block(item, id);
+                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+                g_signal_handler_unblock(item, id);
+            }
+            break;
+
+            default:
+            break;
+        }
+        l = g_list_next(l);
+    }
+}
+
+static gboolean
+gwy_app_data_corner_menu_popup_mouse(GtkWidget *menu,
+                                     GdkEventButton *event,
+                                     GtkWidget *ebox)
+{
+    GtkWidget *window;
+    GwyDataView *data_view;
+
+    if (event->button != 1)
+        return FALSE;
+
+    window = gtk_widget_get_ancestor(ebox, GWY_TYPE_DATA_WINDOW);
+    g_return_val_if_fail(window, FALSE);
+    data_view = gwy_data_window_get_data_view(GWY_DATA_WINDOW(window));
+
+    gwy_app_data_browser_select_data_view(data_view);
+    gwy_app_data_corner_menu_update(menu, data_view);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                   event->button, event->time);
+
+    return FALSE;
+}
+
+static GtkWidget*
+gwy_app_menu_data_corner_create(GtkAccelGroup *accel_group)
+{
+    GtkWidget *menu, *item;
+    GtkRadioMenuItem *r;
+
+    corner_item_quark = g_quark_from_static_string("id");
+
+    menu = gtk_menu_new();
+    if (accel_group)
+        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);
+
+    item = gtk_radio_menu_item_new_with_mnemonic(NULL,
+                                                 _("Pi_xelwise Square"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_object_set_qdata(G_OBJECT(item), corner_item_quark,
+                       GUINT_TO_POINTER(ITEM_PIXELSQUARE));
+    g_signal_connect(item, "activate",
+                     G_CALLBACK(gwy_app_data_window_change_square),
+                     GINT_TO_POINTER(FALSE));
+
+    r = GTK_RADIO_MENU_ITEM(item);
+    item = gtk_radio_menu_item_new_with_mnemonic_from_widget(r,
+                                                             _("_Physically "
+                                                               "Square"));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    g_object_set_qdata(G_OBJECT(item), corner_item_quark,
+                       GUINT_TO_POINTER(ITEM_REALSQUARE));
+    g_signal_connect(item, "activate",
+                     G_CALLBACK(gwy_app_data_window_change_square),
+                     GINT_TO_POINTER(TRUE));
+
+    return menu;
+}
+
+static void
+gwy_app_data_window_change_square(GtkWidget *item,
+                                  gpointer user_data)
+{
+    gboolean realsquare = GPOINTER_TO_INT(user_data);
+    GwyDataView *data_view;
+    GwyContainer *data;
+    const gchar *key;
+    gchar *s;
+
+    if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item))) {
+        gwy_debug("bogus update");
+        return;
+    }
+
+    gwy_debug("new square mode: %s", realsquare ? "Physical" : "Pixelwise");
+    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data,
+                                     GWY_APP_DATA_VIEW, &data_view,
+                                     0);
+    data = gwy_data_view_get_data(data_view);
+    key = gwy_data_view_get_data_prefix(data_view);
+    g_return_if_fail(key);
+    s = g_strconcat(key, "/realsquare", NULL);
+    if (realsquare)
+        gwy_container_set_boolean_by_name(data, s, realsquare);
+    else
+        gwy_container_remove_by_name(data, s);
+    g_free(s);
+}
+
 /*****************************************************************************
  *                                                                           *
  *     Graph window list management                                          *
@@ -371,7 +658,42 @@ _gwy_app_data_window_setup(GwyDataWindow *data_window)
 void
 _gwy_app_graph_window_setup(GwyGraphWindow *graph_window)
 {
+    static GtkWidget *popup_menu = NULL;
+
+    GtkWidget *graph, *main_window;
+    GtkAccelGroup *accel_group;
+
+    if (!popup_menu && (main_window = gwy_app_main_window_get())) {
+        g_return_if_fail(GTK_IS_WINDOW(main_window));
+        accel_group = GTK_ACCEL_GROUP(g_object_get_data(G_OBJECT(main_window),
+                                                        "accel_group"));
+
+        if (accel_group && !popup_menu) {
+            GList *items;
+
+            popup_menu = gwy_app_build_graph_menu(accel_group);
+            items = gtk_container_get_children(GTK_CONTAINER(popup_menu));
+            if (GTK_IS_TEAROFF_MENU_ITEM(items->data))
+                gtk_widget_destroy(GTK_WIDGET(items->data));
+            g_list_free(items);
+            gtk_widget_show_all(popup_menu);
+        }
+    }
+
     gwy_app_add_main_accel_group(GTK_WINDOW(graph_window));
+
+    graph = gwy_graph_window_get_graph(graph_window);
+    g_signal_connect_swapped(graph, "button-press-event",
+                             G_CALLBACK(gwy_app_graph_popup_menu_popup_mouse),
+                             popup_menu);
+    g_signal_connect_swapped(gwy_graph_get_area(GWY_GRAPH(graph)),
+                             "button-press-event",
+                             G_CALLBACK(gwy_app_graph_popup_menu_popup_mouse),
+                             popup_menu);
+    /* FIXME: Graphs don't get keyboard events. */
+    g_signal_connect_swapped(graph, "popup-menu",
+                             G_CALLBACK(gwy_app_graph_popup_menu_popup_key),
+                             popup_menu);
 }
 
 /**
@@ -395,6 +717,48 @@ _gwy_app_graph_set_current(GwyGraph *graph)
     gwy_app_sensitivity_set_state(GWY_MENU_FLAG_GRAPH, GWY_MENU_FLAG_GRAPH);
 }
 
+static gboolean
+gwy_app_graph_popup_menu_popup_mouse(GtkWidget *menu,
+                                     GdkEventButton *event,
+                                     GwyGraph *graph)
+{
+    if (event->button != 3)
+        return FALSE;
+
+    if (GWY_IS_GRAPH_AREA(graph)) {
+        graph = GWY_GRAPH(gtk_widget_get_ancestor(GTK_WIDGET(graph),
+                                                  GWY_TYPE_GRAPH));
+        if (!graph)
+            return FALSE;
+    }
+    gwy_app_data_browser_select_graph(graph);
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                   event->button, event->time);
+
+    return TRUE;
+}
+
+static void
+gwy_app_graph_popup_menu_position(G_GNUC_UNUSED GtkMenu *menu,
+                                 gint *x,
+                                 gint *y,
+                                 gboolean *push_in,
+                                 GtkWidget *widget)
+{
+    gdk_window_get_origin(widget->window, x, y);
+    *push_in = TRUE;
+}
+
+static void
+gwy_app_graph_popup_menu_popup_key(GtkWidget *menu,
+                                   GtkWidget *graph)
+{
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+                   (GtkMenuPositionFunc)gwy_app_graph_popup_menu_position,
+                   graph,
+                   0, gtk_get_current_event_time());
+}
+
 /*****************************************************************************
  *                                                                           *
  *     3D window                                                             *
@@ -658,287 +1022,6 @@ _gwy_app_spectra_set_current(GwySpectra *spectra)
  *                                                                           *
  *****************************************************************************/
 
-static GtkWidget*
-gwy_app_menu_data_popup_create(GtkAccelGroup *accel_group)
-{
-    static struct {
-        const gchar *label;
-        gpointer callback;
-        gpointer cbdata;
-        guint key;
-        GdkModifierType mods;
-    }
-    const menu_items[] = {
-        {
-            NULL, gwy_app_run_process_func,
-            "mask_remove", GDK_K, GDK_CONTROL_MASK
-        },
-        {
-            N_("Mask _Color..."),  gwy_app_change_mask_color_cb,
-            NULL, 0, 0
-        },
-        {
-            NULL, gwy_app_run_process_func,
-            "fix_zero", 0, 0
-        },
-        {
-            NULL, gwy_app_run_process_func,
-            "presentation_remove", GDK_K, GDK_CONTROL_MASK | GDK_SHIFT_MASK
-        },
-        {
-            NULL, gwy_app_run_process_func,
-            "level", GDK_L, GDK_CONTROL_MASK
-        },
-        {
-            N_("Zoom _1:1"), gwy_app_data_window_reset_zoom,
-            NULL, 0, 0
-        },
-    };
-    GwySensitivityGroup *sensgroup;
-    GtkWidget *menu, *item;
-    const gchar *name;
-    guint i, mask;
-
-    menu = gtk_menu_new();
-    if (accel_group)
-        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);
-    sensgroup = gwy_app_sensitivity_get_group();
-    for (i = 0; i < G_N_ELEMENTS(menu_items); i++) {
-        if (menu_items[i].callback == gwy_app_run_process_func
-            && !gwy_process_func_get_run_types((gchar*)menu_items[i].cbdata)) {
-            g_warning("Processing function <%s> for "
-                      "data view context menu is not available.",
-                      (gchar*)menu_items[i].cbdata);
-            continue;
-        }
-        if (menu_items[i].callback == gwy_app_run_process_func) {
-            name = _(gwy_process_func_get_menu_path(menu_items[i].cbdata));
-            name = strrchr(name, '/');
-            if (!name) {
-                g_warning("Invalid translated menu path for <%s>",
-                          (const gchar*)menu_items[i].cbdata);
-                continue;
-            }
-            item = gtk_menu_item_new_with_mnemonic(name + 1);
-            mask = gwy_process_func_get_sensitivity_mask(menu_items[i].cbdata);
-            gwy_sensitivity_group_add_widget(sensgroup, item, mask);
-        }
-        else {
-            item = gtk_menu_item_new_with_mnemonic(_(menu_items[i].label));
-
-            if (menu_items[i].callback == gwy_app_change_mask_color_cb)
-                gwy_sensitivity_group_add_widget(sensgroup, item,
-                                                 GWY_MENU_FLAG_DATA_MASK);
-        }
-
-        if (menu_items[i].key)
-            gtk_widget_add_accelerator(item, "activate", accel_group,
-                                       menu_items[i].key, menu_items[i].mods,
-                                       GTK_ACCEL_VISIBLE | GTK_ACCEL_LOCKED);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect_swapped(item, "activate",
-                                 G_CALLBACK(menu_items[i].callback),
-                                 menu_items[i].cbdata);
-    }
-
-    return menu;
-}
-
-static gboolean
-gwy_app_data_popup_menu_popup_mouse(GtkWidget *menu,
-                                    GdkEventButton *event,
-                                    GwyDataView *data_view)
-{
-    if (event->button != 3)
-        return FALSE;
-
-    gwy_app_data_browser_select_data_view(data_view);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   event->button, event->time);
-
-    return TRUE;
-}
-
-static void
-gwy_app_data_popup_menu_position(G_GNUC_UNUSED GtkMenu *menu,
-                                 gint *x,
-                                 gint *y,
-                                 gboolean *push_in,
-                                 GtkWidget *window)
-{
-    GwyDataView *data_view;
-
-    data_view = gwy_data_window_get_data_view(GWY_DATA_WINDOW(window));
-    gdk_window_get_origin(GTK_WIDGET(data_view)->window, x, y);
-    *push_in = TRUE;
-}
-
-static void
-gwy_app_data_popup_menu_popup_key(GtkWidget *menu,
-                                  GtkWidget *data_window)
-{
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
-                   (GtkMenuPositionFunc)gwy_app_data_popup_menu_position,
-                   data_window,
-                   0, gtk_get_current_event_time());
-}
-
-/**
- * gwy_app_data_corner_menu_update:
- * @menu: Data window corner menu.
- * @data_view: The corresponding data view.
- *
- * Updates corner menu to reflect data window's state before we show it.
- **/
-static void
-gwy_app_data_corner_menu_update(GtkWidget *menu,
-                                GwyDataView *data_view)
-{
-    gboolean realsquare = FALSE;
-    GwyContainer *data;
-    const gchar *key;
-    gchar *s;
-    GtkWidget *item;
-    GList *l;
-    gulong id;
-    guint i;
-
-    /* Square mode */
-    data = gwy_data_view_get_data(data_view);
-    key = gwy_data_view_get_data_prefix(data_view);
-    s = g_strconcat(key, "/realsquare", NULL);
-    gwy_container_gis_boolean_by_name(data, s, &realsquare);
-    gwy_debug("view's realsquare: %d", realsquare);
-    g_free(s);
-
-    /* Update stuff */
-    l = gtk_container_get_children(GTK_CONTAINER(menu));
-    while (l) {
-        item = GTK_WIDGET(l->data);
-        i = GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(item),
-                                                corner_item_quark));
-        switch (i) {
-            case ITEM_PIXELSQUARE:
-            if (!realsquare) {
-                gwy_debug("setting Pixelwise active");
-                id = g_signal_handler_find(item, G_SIGNAL_MATCH_FUNC,
-                                           0, 0, NULL,
-                                           gwy_app_data_window_change_square,
-                                           NULL);
-                g_signal_handler_block(item, id);
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
-                g_signal_handler_unblock(item, id);
-            }
-            break;
-
-            case ITEM_REALSQUARE:
-            if (realsquare) {
-                gwy_debug("setting Physical active");
-                id = g_signal_handler_find(item, G_SIGNAL_MATCH_FUNC,
-                                           0, 0, NULL,
-                                           gwy_app_data_window_change_square,
-                                           NULL);
-                g_signal_handler_block(item, id);
-                gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
-                g_signal_handler_unblock(item, id);
-            }
-            break;
-
-            default:
-            break;
-        }
-        l = g_list_next(l);
-    }
-}
-
-static gboolean
-gwy_app_data_corner_menu_popup_mouse(GtkWidget *menu,
-                                     GdkEventButton *event,
-                                     GtkWidget *ebox)
-{
-    GtkWidget *window;
-    GwyDataView *data_view;
-
-    if (event->button != 1)
-        return FALSE;
-
-    window = gtk_widget_get_ancestor(ebox, GWY_TYPE_DATA_WINDOW);
-    g_return_val_if_fail(window, FALSE);
-    data_view = gwy_data_window_get_data_view(GWY_DATA_WINDOW(window));
-
-    gwy_app_data_browser_select_data_view(data_view);
-    gwy_app_data_corner_menu_update(menu, data_view);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   event->button, event->time);
-
-    return FALSE;
-}
-
-static GtkWidget*
-gwy_app_menu_data_corner_create(GtkAccelGroup *accel_group)
-{
-    GtkWidget *menu, *item;
-    GtkRadioMenuItem *r;
-
-    corner_item_quark = g_quark_from_static_string("id");
-
-    menu = gtk_menu_new();
-    if (accel_group)
-        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);
-
-    item = gtk_radio_menu_item_new_with_mnemonic(NULL,
-                                                 _("Pi_xelwise Square"));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    g_object_set_qdata(G_OBJECT(item), corner_item_quark,
-                       GUINT_TO_POINTER(ITEM_PIXELSQUARE));
-    g_signal_connect(item, "activate",
-                     G_CALLBACK(gwy_app_data_window_change_square),
-                     GINT_TO_POINTER(FALSE));
-
-    r = GTK_RADIO_MENU_ITEM(item);
-    item = gtk_radio_menu_item_new_with_mnemonic_from_widget(r,
-                                                             _("_Physically "
-                                                               "Square"));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    g_object_set_qdata(G_OBJECT(item), corner_item_quark,
-                       GUINT_TO_POINTER(ITEM_REALSQUARE));
-    g_signal_connect(item, "activate",
-                     G_CALLBACK(gwy_app_data_window_change_square),
-                     GINT_TO_POINTER(TRUE));
-
-    return menu;
-}
-
-static void
-gwy_app_data_window_change_square(GtkWidget *item,
-                                  gpointer user_data)
-{
-    gboolean realsquare = GPOINTER_TO_INT(user_data);
-    GwyDataView *data_view;
-    GwyContainer *data;
-    const gchar *key;
-    gchar *s;
-
-    if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item))) {
-        gwy_debug("bogus update");
-        return;
-    }
-
-    gwy_debug("new square mode: %s", realsquare ? "Physical" : "Pixelwise");
-    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data,
-                                     GWY_APP_DATA_VIEW, &data_view,
-                                     0);
-    data = gwy_data_view_get_data(data_view);
-    key = gwy_data_view_get_data_prefix(data_view);
-    g_return_if_fail(key);
-    s = g_strconcat(key, "/realsquare", NULL);
-    if (realsquare)
-        gwy_container_set_boolean_by_name(data, s, realsquare);
-    else
-        gwy_container_remove_by_name(data, s);
-    g_free(s);
-}
-
 /**
  * gwy_app_switch_tool:
  * @toolname: Tool name, that is #GType name of the tool type.
@@ -1258,6 +1341,8 @@ gwy_app_init_common(GError **error,
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GRADIENT));
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GL_MATERIAL));
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GRAIN_VALUE));
+    gwy_resource_class_load(g_type_class_peek(GWY_TYPE_CALIBRATION));
+
 
     /* Load settings */
     settings_file = gwy_app_settings_get_settings_filename();
