@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: data-browser.h 11406 2010-09-22 08:13:22Z yeti-dn $
+ *  @(#) $Id: data-browser.h 11847 2011-01-24 19:50:24Z klapetek $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek, Chris Anderson
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, sidewinderasu@gmail.com.
  *
@@ -58,7 +58,8 @@ typedef enum {
     GWY_DATA_ITEM_RANGE_TYPE,
     GWY_DATA_ITEM_REAL_SQUARE,
     GWY_DATA_ITEM_SELECTIONS,
-    GWY_DATA_ITEM_META
+    GWY_DATA_ITEM_META,
+    GWY_DATA_ITEM_CALDATA
 } GwyDataItem;
 
 typedef enum {
