@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: datachooser.c 11407 2010-09-22 13:20:37Z yeti-dn $
+ *  @(#) $Id: datachooser.c 11752 2011-01-13 14:18:28Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -80,6 +80,13 @@ enum {
     MODEL_NCOLUMNS
 };
 
+/*****************************************/
+GtkTreeModel *gwy_data_chooser_get_filter(GwyDataChooser *chooser)
+{
+    return chooser->filter;
+}
+/******************************************/
+
 static void     gwy_data_chooser_finalize       (GObject *object);
 static void     gwy_data_chooser_destroy        (GtkObject *object);
 static gboolean gwy_data_chooser_is_visible     (GtkTreeModel *model,
@@ -452,7 +459,8 @@ gwy_data_chooser_find_data(GwyDataChooser *chooser,
                            MODEL_COLUMN_CONTAINER, &container2,
                            MODEL_COLUMN_ID, &id2,
                            -1);
-        g_object_unref(container);
+        if (container2)
+            g_object_unref(container2);
         if (container2 == container && id2 == id)
             return TRUE;
         if (container2 > container || (container2 == container && id2 > id))
@@ -545,9 +553,11 @@ gwy_data_chooser_receive_event(GwyContainer *data,
 
     /* Find if we already have an event on the same object. */
     for (item = chooser->events; item; item = g_list_next(item)) {
-        event = (GwyDataChooserEvent*)item->data;
-        if (event->container == data && event->id == id)
+        GwyDataChooserEvent *thisevent = (GwyDataChooserEvent*)item->data;
+        if (thisevent->container == data && thisevent->id == id) {
+            event = thisevent;
             break;
+        }
     }
 
     if (event) {
@@ -555,7 +565,7 @@ gwy_data_chooser_receive_event(GwyContainer *data,
             if (event->event_type == GWY_DATA_WATCH_EVENT_REMOVED)
                 g_warning("Got event REMOVED twice on %p, %d.", data, id);
             else if (event->event_type == GWY_DATA_WATCH_EVENT_ADDED) {
-                /* Got rid of the data altogether, as we do not display it. */
+                /* Get rid of the data altogether, as we do not display it. */
                 chooser->events = g_list_delete_link(chooser->events, item);
                 return;
             }
@@ -563,7 +573,7 @@ gwy_data_chooser_receive_event(GwyContainer *data,
                 event->event_type = event_type;
         }
         else if (event_type == GWY_DATA_WATCH_EVENT_ADDED) {
-            g_warning("Got event ADDED on existing item %p, %d.", data, id);
+            g_warning("Got event ADDED twice on %p, %d.", data, id);
         }
         else if (event_type == GWY_DATA_WATCH_EVENT_CHANGED) {
             if (event->event_type == GWY_DATA_WATCH_EVENT_REMOVED)
