@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyappfilechooser.c 10215 2009-10-05 15:00:59Z yeti-dn $
+ *  @(#) $Id: gwyappfilechooser.c 11403 2010-09-21 12:05:50Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -90,6 +90,7 @@ static void       gwy_app_file_chooser_add_preview    (GwyAppFileChooser *choose
 static void       gwy_app_file_chooser_update_preview (GwyAppFileChooser *chooser);
 static gboolean   gwy_app_file_chooser_do_full_preview(gpointer user_data);
 static void       gwy_app_file_chooser_free_preview   (GwyAppFileChooser *chooser);
+static void       ensure_gtk_recently_used            (void);
 
 G_DEFINE_TYPE(GwyAppFileChooser, _gwy_app_file_chooser,
               GTK_TYPE_FILE_CHOOSER_DIALOG)
@@ -228,6 +229,8 @@ _gwy_app_file_chooser_get(GtkFileChooserAction action)
     if (*instance)
         return *instance;
 
+    ensure_gtk_recently_used();
+
     chooser = g_object_new(GWY_TYPE_APP_FILE_CHOOSER,
                            "title", title,
                            "action", action,
@@ -410,8 +413,6 @@ gwy_app_file_chooser_type_changed(GwyAppFileChooser *chooser,
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
-    gboolean active = FALSE;
-    gchar *key;
 
     if (!gtk_tree_selection_get_selected(selection, &model, &iter))
         return;
@@ -985,4 +986,33 @@ gwy_app_file_chooser_free_preview(GwyAppFileChooser *chooser)
     gwy_object_unref(chooser->preview_data);
 }
 
+/* Work around crashes in the file open dialog in some Gtk+ versions if no
+ * .recently-used.xbel is present. */
+static void
+ensure_gtk_recently_used(void)
+{
+    static gboolean ensured = FALSE;
+    gchar *filename = NULL;
+
+    if (ensured)
+        return;
+
+    /* Gtk+ 2.12 implies GBookmarkFile is also available. */
+#if (GTK_CHECK_VERSION(2,12,0))
+    filename = g_build_filename(g_get_home_dir(), ".recently-used.xbel", NULL);
+    if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
+        GBookmarkFile *bookmarkfile = g_bookmark_file_new();
+        GError *error = NULL;
+
+        if (!g_bookmark_file_to_file(bookmarkfile, filename, &error)) {
+            g_warning("Failed to create %s: %s", filename, error->message);
+            g_clear_error(&error);
+        }
+        g_object_unref(bookmarkfile);
+    }
+#endif
+    g_free(filename);
+    ensured = TRUE;
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
