@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyddion.c 11624 2010-11-29 08:27:32Z yeti-dn $
+ *  @(#) $Id: gwyddion.c 11803 2011-01-19 22:33:27Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -22,12 +22,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyversion.h>
 #include <libgwyddion/gwydebugobjects.h>
 #include <libprocess/gwygrainvalue.h>
+#include <libprocess/gwycalibration.h>
 #include <libgwymodule/gwymoduleloader.h>
 #include <libgwymodule/gwymodule-file.h>
 #include <libgwydgets/gwydgets.h>
@@ -44,16 +46,13 @@
 #define LOG_TO_FILE_DEFAULT FALSE
 #endif
 
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif
-
 typedef struct {
     gboolean no_splash;
     gboolean debug_objects;
     gboolean startup_time;
     gboolean check;
     gboolean log_to_file;
+    gboolean disable_gl;
     GwyAppRemoteType remote;
 } GwyAppOptions;
 
@@ -83,7 +82,8 @@ static void gwy_app_check_version           (void);
 static void sneaking_thread_init            (void);
 
 static GwyAppOptions app_options = {
-    FALSE, FALSE, FALSE, FALSE, LOG_TO_FILE_DEFAULT, GWY_APP_REMOTE_NONE,
+    FALSE, FALSE, FALSE, FALSE, LOG_TO_FILE_DEFAULT, FALSE,
+    GWY_APP_REMOTE_NONE,
 };
 
 int
@@ -141,6 +141,8 @@ main(int argc, char *argv[])
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GL_MATERIAL));
     gwy_app_splash_set_message(_("grain quantities"));
     gwy_resource_class_load(g_type_class_peek(GWY_TYPE_GRAIN_VALUE));
+    gwy_app_splash_set_message(_("calibrations"));
+    gwy_resource_class_load(g_type_class_peek(GWY_TYPE_CALIBRATION));
     gwy_app_splash_set_message_prefix(NULL);
     debug_time(timer, "load resources");
 
@@ -156,9 +158,7 @@ main(int argc, char *argv[])
     gwy_module_register_modules((const gchar**)module_dirs);
     /* The Python initialisation somehow overrides SIGINT and Gwyddion can no
      * longer be terminated with Ctrl-C.  Fix it. */
-#if HAVE_SIGNAL_H
     signal(SIGINT, SIG_DFL);
-#endif
     debug_time(timer, "register modules");
 
     if (app_options.check) {
@@ -294,6 +294,10 @@ process_preinit_options(int *argc,
                 options->log_to_file = FALSE;
                 continue;
             }
+            if (gwy_strequal((*argv)[i], "--disable-gl")) {
+                options->disable_gl = TRUE;
+                continue;
+            }
             if (gwy_strequal((*argv)[i], "--check")) {
                 options->check = TRUE;
                 continue;
@@ -322,6 +326,7 @@ print_help(void)
 "     --remote-new           Load FILES to a running instance or run a new one.\n"
 "     --remote-existing      Load FILES to a running instance or fail.\n"
 "     --check                Check FILES, print problems and terminate.\n"
+"     --disable-gl           Disable OpenGL, including any availability checks.\n"
 "     --log-to-file          Redirect messages file set in GWYDDION_LOGFILE.\n"
 "     --no-log-to-file       Print messages to console.\n"
 "     --debug-objects        Catch leaking objects (devel only).\n"
@@ -457,23 +462,6 @@ setup_locale_from_win32_registry(void)
     DWORD size = sizeof(locale);
     HKEY reg_key;
 
-#ifdef _MSC_VER
-    if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Gwyddion\\1.0"),
-                     0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, TEXT("gwy_locale"), NULL, NULL, locale, &size) == ERROR_SUCCESS){
-            g_setenv("LANG", locale, TRUE);
-            RegCloseKey(reg_key);
-            return;
-        }
-        RegCloseKey(reg_key);
-    }
-    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\Gwyddion\\1.0"),
-                     0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, TEXT("gwy_locale"), NULL, NULL, locale, &size) == ERROR_SUCCESS)
-            g_setenv("LANG", locale, TRUE);
-        RegCloseKey(reg_key);
-    }
-#else
     if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Gwyddion\\2.0"),
                      0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
         if (RegQueryValueEx(reg_key, TEXT("Locale"), NULL, NULL, locale, &size) == ERROR_SUCCESS){
@@ -490,7 +478,6 @@ setup_locale_from_win32_registry(void)
         RegCloseKey(reg_key);
     }
 #endif
-#endif
 }
 
 static void
@@ -590,8 +577,9 @@ gwy_app_init(int *argc,
     if (sizeof(GWY_VERSION_STRING) > 9)
         g_log_set_always_fatal(G_LOG_LEVEL_CRITICAL);
     g_set_application_name(PACKAGE_NAME);
-    gwy_app_gl_init(argc, argv);
-    /* XXX: These reference are never released. */
+    if (!gwy_app_gl_disabled())
+        gwy_app_gl_init(argc, argv);
+    /* XXX: These references are never released. */
     gwy_data_window_class_set_tooltips(gwy_app_get_tooltips());
     gwy_3d_window_class_set_tooltips(gwy_app_get_tooltips());
     gwy_graph_window_class_set_tooltips(gwy_app_get_tooltips());
@@ -658,4 +646,10 @@ sneaking_thread_init(void)
     g_module_close(main_module);
 }
 
+gboolean
+gwy_app_gl_disabled(void)
+{
+    return app_options.disable_gl;
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
