@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: meta.c 7955 2007-04-30 18:18:43Z miraval $
+ *  @(#) $Id: meta.c 10256 2009-10-16 12:59:47Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -26,6 +26,7 @@
 #include <libprocess/datafield.h>
 #include <libgwydgets/gwydatawindow.h>
 #include <libgwydgets/gwydgetutils.h>
+#include <app/app.h>
 #include <app/data-browser.h>
 #include <app/gwymoduleutils.h>
 
@@ -114,6 +115,7 @@ gwy_app_metadata_browser(GwyContainer *data,
                                 MIN(request.width + 24, 2*gdk_screen_width()/3),
                                 MIN(request.height + 32,
                                     2*gdk_screen_height()/3));
+    gwy_app_add_main_accel_group(GTK_WINDOW(browser->window));
 
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(browser->window), vbox);
@@ -424,8 +426,14 @@ gwy_meta_browser_add_line(gpointer hkey,
         fd->value = (gchar*)val;
         fd->isok = TRUE;
     }
-    else if ((s = g_locale_to_utf8(val, -1, NULL, NULL, NULL)))
+    else if ((s = g_convert(val, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL)))
         fd->value = s;
+    else {
+        g_warning("Bogus metadata <%s> at key <%s>",
+                  val, g_quark_to_string(quark));
+        g_free(fd);
+        return;
+    }
 
     /* The same applies to markup validity.  Fix invalid markup by taking it
      * literally. */
