@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: settings.c 9274 2009-06-17 13:37:48Z yeti-dn $
+ *  @(#) $Id: settings.c 11681 2010-12-10 07:56:23Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -22,16 +22,8 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-
-#if (defined(HAVE_SYS_STAT_H) || defined(_WIN32))
 #include <sys/stat.h>
-/* And now we are in a deep s... */
-#endif
-
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-
 #include <glib/gstdio.h>
 
 #ifdef HAVE_GTKGLEXT
