@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: undo.c 11376 2010-08-30 12:49:19Z yeti-dn $
+ *  @(#) $Id: undo.c 11930 2011-01-31 20:34:41Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -28,6 +28,7 @@
 #include <libprocess/datafield.h>
 #include <app/menu.h>
 #include <app/undo.h>
+#include "gwyappinternal.h"
 
 enum {
     UNDO_LEVELS = 3
@@ -189,6 +190,62 @@ gwy_app_undo_qcheckpoint(GwyContainer *data,
     return id;
 }
 
+/* Create a list of calibration keys corresponding to channel keys in @keys.
+ * If @only_calib_keys is true then only these keys are returned
+ * otherwise a list that contains all original keys plus any missing
+ * calibration keys is returned. */
+static GQuark*
+construct_calibrations_keys(GwyContainer *data,
+                            guint *pn, const GQuark *keys,
+                            gboolean only_calib_keys)
+{
+    static const gchar *calib_keys[] = {
+        "cal_xerr", "cal_yerr", "cal_zerr", "cal_xunc", "cal_yunc", "cal_zunc",
+    };
+
+    GArray *newkeys;
+    guint n = *pn;
+    GwyAppKeyType type;
+    gint len, i, j, k, id;
+
+    newkeys = g_array_new(FALSE, FALSE, sizeof(GQuark));
+    for (i = 0; i < n; i++) {
+        GQuark key = keys[i];
+        if (!key)
+            continue;
+
+        id = _gwy_app_analyse_data_key(g_quark_to_string(key), &type, &len);
+        if (!only_calib_keys)
+            g_array_append_val(newkeys, key);
+        if (type != KEY_IS_DATA)
+            continue;
+
+        for (k = 0; k < G_N_ELEMENTS(calib_keys); k++) {
+            gchar buf[40];
+            GQuark calkey;
+
+            g_snprintf(buf, sizeof(buf), "/%d/data/%s", id, calib_keys[k]);
+            calkey = g_quark_from_string(buf);
+            if (!gwy_container_contains(data, calkey))
+                continue;
+
+            if (only_calib_keys)
+                g_array_append_val(newkeys, calkey);
+            else {
+                for (j = 0; j < n; j++) {
+                    if (keys[j] == calkey)
+                        break;
+                }
+                if (j == n)
+                    g_array_append_val(newkeys, calkey);
+            }
+        }
+    }
+
+    *pn = newkeys->len;
+    return (GQuark*)g_array_free(newkeys, FALSE);
+}
+
 /**
  * gwy_app_undo_qcheckpointv:
  * @data: A data container.
@@ -202,6 +259,12 @@ gwy_app_undo_qcheckpoint(GwyContainer *data,
  * In addition to what gwy_undo_checkpointv() does, this function takes care
  * of updating application controls state.
  *
+ * If @keys include channel keys the corresponding channel calibration data is
+ * saved together with the data and then removed from the container.  This
+ * prevents mismatch between a data field and its calibration data.  Therefore,
+ * if you perform a data operation that can also meaningfully transform the
+ * corresponding calibration data take care to obtain it before saving undo.
+ *
  * Returns: Undo level id.  Not useful (yet).
  **/
 gulong
@@ -209,13 +272,25 @@ gwy_app_undo_qcheckpointv(GwyContainer *data,
                           guint n,
                           const GQuark *keys)
 {
+    GQuark *calkeys;
     gulong id;
+    guint nwcal, i;
+
+    nwcal = n;
+    calkeys = construct_calibrations_keys(data, &nwcal, keys, FALSE);
+    id = gwy_undo_qcheckpointv(data, nwcal, calkeys);
+    g_free(calkeys);
 
-    id = gwy_undo_qcheckpointv(data, n, keys);
     if (id)
         gwy_app_sensitivity_set_state(GWY_MENU_FLAG_UNDO | GWY_MENU_FLAG_REDO,
                                       GWY_MENU_FLAG_UNDO);
 
+    nwcal = n;
+    calkeys = construct_calibrations_keys(data, &nwcal, keys, TRUE);
+    for (i = 0; i < nwcal; i++)
+        gwy_container_remove(data, calkeys[i]);
+    g_free(calkeys);
+
     return id;
 }
 
@@ -470,6 +545,7 @@ gwy_undo_qcheckpointv(GwyContainer *data,
 
     return level->id;
 }
+
 /**
  * gwy_app_undo_reuse_levels:
  * @level: An undo level with objects that have to be either duplicated or
@@ -934,6 +1010,34 @@ gwy_undo_container_remove(GwyContainer *data,
     appundo->redo = gwy_undo_container_remove_from_list(appundo->redo, prefix);
 }
 
+/**
+ * gwy_app_undo_container_remove:
+ * @data: A data container managed by the data-browser.
+ * @prefix: Prefix to remove undo/redo information under.  Pass %NULL to remove
+ *          undo/redo information altogether.
+ *
+ * Removes undo/redo information for a data container.
+ *
+ * In addition to what gwy_undo_container_remove() does, this function takes
+ * care of updating application controls state.
+ *
+ * Since: 2.23
+ **/
+void
+gwy_app_undo_container_remove(GwyContainer *data,
+                              const gchar *prefix)
+{
+    GwyAppUndo *appundo;
+
+    gwy_undo_container_remove(data, prefix);
+    appundo = gwy_undo_get_for_data(data, FALSE);
+    if (!appundo)
+        return;
+    gwy_app_sensitivity_set_state(GWY_MENU_FLAG_UNDO | GWY_MENU_FLAG_REDO,
+                                  (appundo->undo ? GWY_MENU_FLAG_UNDO : 0)
+                                  | (appundo->redo ? GWY_MENU_FLAG_REDO : 0));
+}
+
 /**
  * gwy_undo_get_enabled:
  *
