@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwymath.h 11547 2010-11-15 10:15:09Z yeti-dn $
+ *  @(#) $Id: gwymath.h 11752 2011-01-13 14:18:28Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -95,6 +95,8 @@ gdouble  gwy_math_median               (gsize n,
                                         gdouble *array);
 void     gwy_math_sort                 (gsize n,
                                         gdouble *array);
+gdouble  gwy_math_median_uncertainty   (gsize n, gdouble *array, 
+                                        gdouble *uarray);
 
 G_END_DECLS
 
