@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwynlfitpreset.c 9222 2009-06-08 19:15:54Z miraval $
+ *  @(#) $Id: gwynlfitpreset.c 10342 2009-11-02 22:30:19Z yeti-dn $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -48,7 +48,7 @@ gauss_func(gdouble x,
     }
     *fres = TRUE;
     c = (x - b[0])/b[3];
-    return b[2] * exp(-c*c/2) + b[1];
+    return b[2] * exp(-c*c) + b[1];
 }
 
 static void
@@ -76,7 +76,7 @@ gauss_guess(gint n_dat,
     }
     param[2] -= param[1];
     param[0] = x[imax];
-    param[3] = (x[n_dat-1] - x[0])/4;
+    param[3] = (x[n_dat-1] - x[0])/2/G_SQRT2;
 
     *fres = TRUE;
 }
