@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyutils.c 11586 2010-11-18 15:21:00Z yeti-dn $
+ *  @(#) $Id: gwyutils.c 11744 2011-01-11 08:26:44Z yeti-dn $
  *  Copyright (C) 2003-2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -506,6 +506,7 @@ gwy_osx_find_dir_in_bundle(const gchar *dirname)
             basedir = malloc(len + 1);
             strncpy(basedir, res_url_path, len);
             basedir[len] = 0;
+            free(res_url_path);
         }
         if (res_url_ref)
             CFRelease(res_url_ref);
@@ -546,17 +547,6 @@ gwy_osx_find_dir_in_bundle(const gchar *dirname)
 gchar*
 gwy_find_self_dir(const gchar *dirname)
 {
-#ifdef _MSC_VER
-    static gchar *basedir = NULL;
-
-    if (!basedir)
-        basedir = g_win32_get_package_installation_directory(NULL, dll_name);
-    if (gwy_strequal(dirname, "data"))
-        dirname = NULL;
-
-    return g_build_filename(basedir, dirname, NULL);
-#endif    /* _MSC_VER */
-
 #ifdef G_OS_UNIX
 #ifdef GWY_LIBDIR
     static const struct {
@@ -622,7 +612,6 @@ gwy_find_self_dir(const gchar *dirname)
 #endif    /* G_OS_UNIX */
 
 #ifdef G_OS_WIN32
-#ifndef _MSC_VER
     static const struct {
         const gchar *id;
         const gchar *env;
@@ -681,7 +670,6 @@ gwy_find_self_dir(const gchar *dirname)
                   dirname, topdir, paths[i].base, paths[i].dir);
         return g_build_filename(topdir, paths[i].base, paths[i].dir, NULL);
     }
-#endif    /* !_MSC_VER */
 #endif    /* G_OS_WIN32 */
 
     g_critical("Cannot find directory for `%s'", dirname);
