@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwycurve.c 7101 2006-11-30 11:53:57Z yeti-dn $
+ *  @(#) $Id: gwycurve.c 8874 2008-12-28 12:16:22Z yeti-dn $
  *  Copyright (C) 2005 Chris Anderson.
  *  E-mail: sidewinder.asu@gmail.com.
  *
@@ -1374,12 +1374,12 @@ gwy_curve_get_vector(GwyCurve *c, gint c_index, gint veclen, gdouble vector[])
             for (x = 0; x < veclen; ++x, rx += dx) {
                 vector[x] = unproject(RADIUS + c->height -
                                       channel->points[(int)rx].y,
-                c->min_y, c->max_y,
-                c->height);
+                                      c->min_y, c->max_y,
+                                      c->height);
             }
         }
         else
-            memset(vector, 0, veclen * sizeof(vector[0]));
+            gwy_clear(vector, veclen);
         break;
     }
 }
