@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwygraphmodel.c 8884 2008-12-28 12:17:44Z yeti-dn $
+ *  @(#) $Id: gwygraphmodel.c 11752 2011-01-13 14:18:28Z yeti-dn $
  *  Copyright (C) 2004-2007 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1481,7 +1481,8 @@ gwy_graph_model_get_ranges(GwyGraphModel *gmodel,
     xmax = ymax = -G_MAXDOUBLE;
     for (i = 0; i < gmodel->curves->len; i++) {
         gcmodel = g_ptr_array_index(gmodel->curves, i);
-        if (gwy_graph_curve_model_get_ranges(gcmodel, x_logscale, y_logscale,
+        if (gcmodel->mode != GWY_GRAPH_CURVE_HIDDEN &&
+            gwy_graph_curve_model_get_ranges(gcmodel, x_logscale, y_logscale,
                                              &cxmin, &cxmax, &cymin, &cymax)) {
             ok |= ALL;
             xmin = MIN(cxmin, xmin);
