@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwynullstore.c 7738 2007-03-18 20:17:53Z yeti-dn $
+ *  @(#) $Id: gwynullstore.c 12125 2011-02-25 20:40:26Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -219,7 +219,7 @@ gwy_null_store_iter_children(GtkTreeModel *model,
     GwyNullStore *store;
 
     store = GWY_NULL_STORE(model);
-    if (!parent || !store->n)
+    if (parent || !store->n)
         return FALSE;
 
     iter->stamp = store->stamp;
