@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwystock.h 9246 2009-06-14 19:10:59Z yeti-dn $
+ *  @(#) $Id: gwystock.h 11519 2010-11-09 09:53:32Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -90,10 +90,14 @@ G_BEGIN_DECLS
 #define GWY_STOCK_MASK_EDITOR           "gwy_mask_editor"
 #define GWY_STOCK_MASK_EXCLUDE          "gwy_mask_exclude"
 #define GWY_STOCK_MASK_EXCLUDE_CIRCLE   "gwy_mask_exclude_circle"
+#define GWY_STOCK_MASK_FILL_DRAW        "gwy_mask_fill_draw"
+#define GWY_STOCK_MASK_FILL_ERASE       "gwy_mask_fill_erase"
 #define GWY_STOCK_MASK_GROW             "gwy_mask_grow"
 #define GWY_STOCK_MASK_INTERSECT        "gwy_mask_intersect"
 #define GWY_STOCK_MASK_INVERT           "gwy_mask_invert"
 #define GWY_STOCK_MASK_LINE             "gwy_mask_line"
+#define GWY_STOCK_MASK_PAINT_DRAW       "gwy_mask_paint_draw"
+#define GWY_STOCK_MASK_PAINT_ERASE      "gwy_mask_paint_erase"
 #define GWY_STOCK_MASK_RECT_EXCLUSIVE   "gwy_mask_rect_exclusive"
 #define GWY_STOCK_MASK_RECT_INCLUSIVE   "gwy_mask_rect_inclusive"
 #define GWY_STOCK_MASK_REMOVE           "gwy_mask_remove"
