@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: cdline.c 9045 2009-03-10 12:00:57Z yeti-dn $
+ *  @(#) $Id: cdline.c 12162 2011-03-02 19:37:49Z klapetek $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -26,6 +26,7 @@
 #include <libgwyddion/gwydebugobjects.h>
 #include "gwyprocessinternal.h"
 #include <libgwyddion/gwynlfit.h>
+#include <libprocess/gwycaldata.h>
 
 typedef gdouble (*GwyCDLineFitFunc)(gdouble x,
                                     gint n_param,
@@ -267,8 +268,8 @@ cd_stepheight(const gdouble *x,
     gint nstep;
     gdouble max, min, val;
     gint imax, imin, iwidth;
-    gint nout;
-
+    gint nout, na, nb;
+    gdouble suma, sumb;
 
     nstep = n_dat/20;
     if (nstep < 1)
@@ -298,14 +299,15 @@ cd_stepheight(const gdouble *x,
     iwidth = imin - imax;
 
     /*FIXME: modified now (imax+iwidth/3, imin-iwidth/3) */
+   // if (!cdata) {
     get_linestatpars(y, n_dat, imax + iwidth/3, imin - iwidth/3, param + 2,
                      err + 2);
 
     param[1] = err[1] = 0;
     nout = 0;
     for (i = 0; i < n_dat; i++) {
-        if ((i < (imax - iwidth/3) && i > (imax - iwidth))    /* /3 */
-            || (i > (imin + iwidth/3) && i < (imin + iwidth))) {       /* /3 */
+        if ((i < (imax - iwidth/3) && i > (imax - iwidth))    // /3 
+            || (i > (imin + iwidth/3) && i < (imin + iwidth))) {       // /3 
             param[1] += y[i];
             err[1] += y[i] * y[i];
             nout++;
@@ -313,11 +315,51 @@ cd_stepheight(const gdouble *x,
     }
     err[1] = sqrt(fabs(err[1] - param[1] * param[1]/nout)/nout);
     param[1] /= (gdouble)nout;
+    err[3] = err[4] = -1;
+    /* } else {
+        param[1] = 0;
+        param[2] = 0;
+        suma = sumb = 0;
+        for (i = 0; i < n_dat; i++) {
+            if ((i < (imax - iwidth/3) && i > (imax - iwidth))   
+                || (i > (imin + iwidth/3) && i < (imin + iwidth))) {      
+                param[1] += y[i]/cdata->zunc[i]/cdata->zunc[i];
+                suma += (1/cdata->zunc[i]/cdata->zunc[i]);
+            } 
+            else if (i>(imax + iwidth/3) && i<=(imin - iwidth/3))
+            {
+                param[2] += y[i]/cdata->zunc[i]/cdata->zunc[i];
+                sumb += (1/cdata->zunc[i]/cdata->zunc[i]);
+            }
+        }
+        param[1]/=suma;
+        param[2]/=sumb;
+
+        err[1] = 0;
+        err[2] = 0;
+        na = nb = 0;
+        for (i = 0; i < n_dat; i++) {
+            if ((i < (imax - iwidth/3) && i > (imax - iwidth))    
+                || (i > (imin + iwidth/3) && i < (imin + iwidth))) {     
+                err[1] += cdata->zunc[i]*cdata->zunc[i];
+                na++;
+            } 
+            else if (i>(imax + iwidth/3) && i<=(imin - iwidth/3))
+            {
+                printf("%g\n", cdata->zunc[i]);
+                err[2] += cdata->zunc[i]*cdata->zunc[i];
+                nb++;
+            }
+        }
+        err[1] = sqrt(err[1]/na);
+        err[2] = sqrt(err[2]/nb);
+        err[3] = cdata->xunc[imax];
+        err[4] = cdata->xunc[imin];
+    }
+    */
 
     param[0] = param[2] - param[1];
-
     err[0] = hypot(err[2], err[1]);
-    err[3] = err[4] = -1;
 
     *fres = TRUE;
 }
@@ -723,13 +765,41 @@ gwy_cdline_fit(GwyCDLine* cdline,
                gdouble *params,
                gdouble *err,
                G_GNUC_UNUSED const gboolean *fixed_param,
-               G_GNUC_UNUSED gpointer user_data)
+               gpointer user_data)
 {
     gboolean fres;
 
     g_return_if_fail(GWY_IS_CDLINE(cdline));
     fres = TRUE;
-    cdline->builtin->function_fit(x, y, n_dat, params, err, NULL, &fres);
+    cdline->builtin->function_fit(x, y, n_dat, params, err, user_data, &fres);
+}
+
+/**
+ * gwy_cdline_fit:
+ * @cdline: A critical dimension evaluator.
+ * @n_dat: The number of data points (number of items in @x and @y).
+ * @x: Abscissa points.
+ * @y: Ordinate points.
+ * @params: Array to store fitted parameter values to.
+ * @err: Array to store parameter errros to, may be %NULL.
+ * @cdata: Curve calibration data, may be %NULL.
+ *
+ * Performs a critical dimension evaulation (fit), allowing user to pass uncertainties.
+ **/
+void         
+gwy_cdline_fit_with_caldata (GwyCDLine* cdline,
+                             gint n_dat,
+                             const gdouble *x,
+                             const gdouble *y,
+                             gdouble *params,
+                             gdouble *err,
+                             GwyCurveCalibrationData *cdata)
+{
+    gboolean fres;
+
+    g_return_if_fail(GWY_IS_CDLINE(cdline));
+    fres = TRUE;
+    cdline->builtin->function_fit(x, y, n_dat, params, err, cdata, &fres);
 }
 
 static void
