@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: dataline.c 11027 2010-01-27 08:50:00Z yeti-dn $
+ *  @(#) $Id: dataline.c 12207 2011-03-11 21:30:27Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -160,6 +160,7 @@ gwy_data_line_new_alike(GwyDataLine *model,
 
     data_line->res = model->res;
     data_line->real = model->real;
+    data_line->off = model->off;
     if (nullme)
         data_line->data = g_new0(gdouble, data_line->res);
     else
@@ -361,6 +362,7 @@ gwy_data_line_clone_real(GObject *source, GObject *copy)
         clone->data = g_renew(gdouble, clone->data, clone->res);
     }
     clone->real = data_line->real;
+    clone->off = data_line->off;
     memcpy(clone->data, data_line->data, data_line->res*sizeof(gdouble));
 
     /* SI Units can be NULL */
