@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: stats.h 10201 2009-10-05 08:25:24Z yeti-dn $
+ *  @(#) $Id: stats.h 11647 2010-12-02 10:43:26Z yeti-dn $
  *  Copyright (C) 2003-2009 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -304,6 +304,16 @@ void    gwy_data_field_area_2dacf           (GwyDataField *data_field,
                                              gint yrange);
 void    gwy_data_field_2dacf                (GwyDataField *data_field,
                                              GwyDataField *target_field);
+void    gwy_data_field_area_racf            (GwyDataField *data_field,
+                                             GwyDataLine *target_line,
+                                             gint col,
+                                             gint row,
+                                             gint width,
+                                             gint height,
+                                             gint nstats);
+void    gwy_data_field_racf                 (GwyDataField *data_field,
+                                             GwyDataLine *target_line,
+                                             gint nstats);
 void    gwy_data_field_area_minkowski_volume(GwyDataField *data_field,
                                              GwyDataLine *target_line,
                                              gint col, gint row,
