@@ -1,5 +1,5 @@
 /*
- *  $Id: aistfile.c 11356 2010-07-10 12:36:43Z yeti-dn $
+ *  $Id: aistfile.c 11681 2010-12-10 07:56:23Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -48,11 +48,7 @@
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
-
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
-
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
 #include <libgwyddion/gwymath.h>
