@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: asciiexport.c 10211 2009-10-05 13:26:46Z yeti-dn $
+ *  @(#) $Id: asciiexport.c 11681 2010-12-10 07:56:23Z yeti-dn $
  *  Copyright (C) 2003,2007 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <locale.h>
 #include <string.h>
+#include <unistd.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <libgwyddion/gwymacros.h>
@@ -38,10 +39,6 @@
 #include <libgwydgets/gwydgetutils.h>
 #include <app/gwyapp.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include "err.h"
 
 #define EXTENSION ".txt"
