@@ -1,5 +1,5 @@
 /*
- *  $Id: nanoeducator.c 11412 2010-09-30 18:57:28Z dn2010 $
+ *  $Id: nanoeducator.c 12271 2011-04-07 19:43:19Z dn2010 $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -945,6 +945,7 @@ make_fd_spectrum(gint res, gdouble xy_step, const gint16 *d16, gboolean flip)
     gdouble *data;
     gint j, amin;
     gdouble z0;
+    gint16 v, a;
 
     dline = gwy_data_line_new(res, xy_step*res, FALSE);
     siunitx = gwy_si_unit_new("m");
@@ -960,8 +961,6 @@ make_fd_spectrum(gint res, gdouble xy_step, const gint16 *d16, gboolean flip)
     /* XXX: The odd coordinates are abscissas.  We only use the zeroth for
      * setting the offset.  If they are not equidistant, though luck... */
     for (j = 0; j < res; j++) {
-        gint v, a;
-
         if (flip) {
             v = d16[2*(res-1 - j)];
             a = d16[2*(res-1 - j) + 1];
@@ -980,7 +979,8 @@ make_fd_spectrum(gint res, gdouble xy_step, const gint16 *d16, gboolean flip)
         }
     }
     gwy_data_line_multiply(dline, 1.0/z0);
-    gwy_data_line_set_offset(dline, xy_step*d16[flip ? 2*(res-1) + 1 : 1]);
+    gwy_data_line_set_offset(dline, xy_step *
+                         GINT16_FROM_LE(d16[flip ? 2*(res-1) + 1 : 1]));
 
     return dline;
 }
@@ -993,7 +993,7 @@ nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
                         gdouble xscale, gdouble yscale,
                         GError **error)
 {
-    gint i, n, speccount, pointstep;
+    gint i, j, n, pointstep;
     GwySpectra *spectra;
     GwyDataLine *dline;
     GwySIUnit *siunit;
@@ -1023,27 +1023,27 @@ nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
     /* For FD curves, there are always two spectra: forward and backward.
      * The backward one is really stored backwards, so we revert it upon
      * reading. */
-    for (i = speccount = 0; speccount < nspectra; i++) {
+    for (i = 0; i < nspectra; i++) {
         x = xscale*GINT16_FROM_LE(p16[pointstep*i]);
         y = yreal - yscale*GINT16_FROM_LE(p16[pointstep*i + 1]);
         n = (pointstep == 3) ? GINT16_FROM_LE(p16[pointstep*i + 2]) : 1;
         gwy_debug("FD spec%d [%g,%g] %dpts", i, x, y, n);
 
-        while (n--) {
+        for (j = 0; j < n; j++) {
             /* Forward */
             dline = make_fd_spectrum(res, xy_step,
-                                     d16 + 2*2*speccount*res, FALSE);
+                                     d16 + 4*(i*n+j)*res,
+                                     FALSE);
             data = gwy_data_line_get_data(dline);
             gwy_spectra_add_spectrum(spectra, dline, x, y);
             g_object_unref(dline);
 
             /* Backward */
             dline = make_fd_spectrum(res, xy_step,
-                                     d16 + 2*2*speccount*res + 2*res, TRUE);
+                                     d16 + 4*(i*n+j)*res + 2*res,
+                                     TRUE);
             gwy_spectra_add_spectrum(spectra, dline, x, y);
             g_object_unref(dline);
-
-            speccount++;
         }
     }
 
@@ -1052,15 +1052,17 @@ nanoedu_read_fd_spectra(const guchar *pos_buffer, gsize pos_size,
 
 static GwyDataLine*
 make_iv_spectrum(gint res, gdouble xy_step,
-                 const gint16 *d16, gdouble q,
-                 gboolean flip)
+                 const gint16 *d16, gdouble q)
 {
     GwyDataLine *dline;
     GwySIUnit *siunitx, *siunity;
     gdouble *data;
     gint j;
+    gint16 v;
 
-    dline = gwy_data_line_new(res, xy_step*fabs(d16[2*(res - 1)] - d16[0]),
+    dline = gwy_data_line_new(res, xy_step*
+                              fabs(GINT16_FROM_LE(d16[2*(res - 1)]) -
+                                   GINT16_FROM_LE(d16[0])),
                               FALSE);
     siunitx = gwy_si_unit_new("V");
     siunity = gwy_si_unit_new("A");
@@ -1073,16 +1075,10 @@ make_iv_spectrum(gint res, gdouble xy_step,
     /* XXX: The even coordinates are abscissas.  We only use the zeroth for
      * setting the offset.  If they are not equidistant, though luck... */
     for (j = 0; j < res; j++) {
-        gint v;
-
-        if (flip)
-            v = d16[2*(res-1 - j) + 1];
-        else
-            v = d16[2*j + 1];
-
+        v = d16[2*j+1];
         data[j] = q*GINT16_FROM_LE(v);
     }
-    gwy_data_line_set_offset(dline, xy_step*d16[flip ? 2*(res-1) : 0]);
+    gwy_data_line_set_offset(dline, xy_step*GINT16_FROM_LE(d16[0]));
 
     return dline;
 }
@@ -1096,7 +1092,7 @@ nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
                         gdouble vscale,
                         GError **error)
 {
-    gint i, n, speccount, pointstep;
+    gint i, j, n, pointstep;
     GwySpectra *spectra;
     GwyDataLine *dline;
     GwySIUnit *siunit;
@@ -1117,32 +1113,22 @@ nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
     g_object_unref(siunit);
     gwy_spectra_set_title(spectra, _("I-V spectra"));
 
-    /* For IV curves, there are always two spectra: forward and backward.
-     * The backward one is really stored backwards, so we revert it upon
-     * reading. */
-    for (i = speccount = 0; speccount < nspectra; i++) {
+    for (i = 0; i < nspectra; i++) {
         x = xscale*GINT16_FROM_LE(p16[pointstep*i]);
         y = yreal - yscale*GINT16_FROM_LE(p16[pointstep*i + 1]);
         n = (pointstep == 3) ? GINT16_FROM_LE(p16[pointstep*i + 2]) : 1;
         gwy_debug("IV spec%d [%g,%g] %dpts", i, x, y, n);
 
-        while (n--) {
-            /* Forward */
+        if (err_SIZE_MISMATCH(error, 2*n*nspectra*res, data_size, FALSE))
+            return NULL;
+
+        for (j = 0; j < n; j++) {
             dline = make_iv_spectrum(res, vscale,
-                                     d16 + 2*2*speccount*res, 1e-12,
-                                     FALSE);
+                                     d16 + 2*(i*n+j)*res,
+                                     1e-12);
             data = gwy_data_line_get_data(dline);
             gwy_spectra_add_spectrum(spectra, dline, x, y);
             g_object_unref(dline);
-
-            /* Backward */
-            dline = make_iv_spectrum(res, vscale,
-                                     d16 + 2*2*speccount*res + 2*res, 1e-12,
-                                     FALSE);
-            gwy_spectra_add_spectrum(spectra, dline, x, y);
-            g_object_unref(dline);
-
-            speccount++;
         }
     }
 
@@ -1151,15 +1137,17 @@ nanoedu_read_iv_spectra(const guchar *pos_buffer, gsize pos_size,
 
 static GwyDataLine*
 make_iz_spectrum(gint res, gdouble xy_step,
-                 const gint16 *d16, gdouble q,
-                 gboolean flip)
+                 const gint16 *d16, gdouble q)
 {
     GwyDataLine *dline;
     GwySIUnit *siunitx, *siunity;
     gdouble *data;
     gint j;
+    gint16 v;
+    gdouble xstep;
 
-    dline = gwy_data_line_new(res, xy_step*res, FALSE);
+    xstep = GINT16_FROM_LE(d16[2*res-1])-GINT16_FROM_LE(d16[1]);
+    dline = gwy_data_line_new(res, xy_step*xstep, FALSE);
     siunitx = gwy_si_unit_new("m");
     siunity = gwy_si_unit_new("A");
     gwy_data_line_set_si_unit_x(dline, siunitx);
@@ -1168,19 +1156,13 @@ make_iz_spectrum(gint res, gdouble xy_step,
     g_object_unref(siunity);
 
     data = gwy_data_line_get_data(dline);
-    /* XXX: The odd coordinates are abscissas.  We only use the zeroth for
-     * setting the offset.  If they are not equidistant, though luck... */
+    /* XXX: The odd coordinates are abscissas.  We use the zeroth and last 
+     * for x offset and scale. If they are not equidistant, though luck... */
     for (j = 0; j < res; j++) {
-        gint v;
-
-        if (flip)
-            v = d16[2*(res-1 - j)];
-        else
-            v = d16[2*j];
-
+        v = d16[2*j];
         data[j] = q*GINT16_FROM_LE(v);
     }
-    gwy_data_line_set_offset(dline, xy_step*d16[flip ? 2*(res-1) + 1 : 1]);
+    gwy_data_line_set_offset(dline, xy_step*GINT16_FROM_LE(d16[1]));
 
     return dline;
 }
@@ -1193,7 +1175,7 @@ nanoedu_read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
                         gdouble xscale, gdouble yscale,
                         GError **error)
 {
-    gint i, n, speccount, pointstep;
+    gint i, j, n, pointstep;
     GwySpectra *spectra;
     GwyDataLine *dline;
     GwySIUnit *siunit;
@@ -1214,25 +1196,20 @@ nanoedu_read_iz_spectra(const guchar *pos_buffer, gsize pos_size,
     g_object_unref(siunit);
     gwy_spectra_set_title(spectra, _("I-Z spectra"));
 
-    /* For IV curves, there are always two spectra: forward and backward.
-     * The backward one is really stored backwards, so we revert it upon
-     * reading. */
-    for (i = speccount = 0; speccount < nspectra; i++) {
+    for (i = 0; i < nspectra; i++) {
         x = xscale*GINT16_FROM_LE(p16[pointstep*i]);
         y = yreal - yscale*GINT16_FROM_LE(p16[pointstep*i + 1]);
         n = (pointstep == 3) ? GINT16_FROM_LE(p16[pointstep*i + 2]) : 1;
         gwy_debug("IZ spec%d [%g,%g] %dpts", i, x, y, n);
 
-        while (n--) {
+        for(j = 0; j < n; j++) {
             /* Only one direction */
             dline = make_iz_spectrum(res, xy_step,
-                                     d16 + 2*speccount*res, 1e-12,
-                                     FALSE);
+                                     d16 + 2*(i*n+j)*res,
+                                     1e-12);
             data = gwy_data_line_get_data(dline);
             gwy_spectra_add_spectrum(spectra, dline, x, y);
             g_object_unref(dline);
-
-            speccount++;
         }
     }
 
