@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: nt-mdt.c 11456 2010-10-25 09:47:26Z yeti-dn $
+ *  @(#) $Id: nt-mdt.c 12264 2011-04-06 18:07:48Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -657,7 +657,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports NT-MDT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.14",
+    "0.15",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -1713,10 +1713,10 @@ extract_mda_data(MDTMDAFrame * dataframe)
 {
     GwyDataField *dfield;
     gdouble *data, *end_data;
-    gdouble xreal, yreal, zscale;
+    gdouble xreal, yreal, zscale, zoffset;
     gint power10xy, power10z;
     GwySIUnit *siunitxy, *siunitz;
-    gint total;
+    gint  nx,ny,total;
     const guchar *p;
     const gchar *cunit;
     gchar *unit;
@@ -1751,16 +1751,15 @@ extract_mda_data(MDTMDAFrame * dataframe)
     }
     gwy_debug("z unit power %d", power10xy);
 
-    xreal = pow10(power10xy) * xAxis->scale;
-    yreal = pow10(power10xy) * yAxis->scale;
+    nx    = xAxis->maxIndex - xAxis->minIndex + 1;
+    ny    = yAxis->maxIndex - yAxis->minIndex + 1;	
+    xreal = pow10(power10xy) * xAxis->scale * (nx - 1);
+    yreal = pow10(power10xy) * yAxis->scale * (ny - 1);
     zscale = pow10(power10z) * zAxis->scale;
-
-    dfield = gwy_data_field_new(xAxis->maxIndex - xAxis->minIndex + 1,
-                                yAxis->maxIndex - yAxis->minIndex + 1,
-                                xreal, yreal, FALSE);
-    total
-        = (xAxis->maxIndex - xAxis->minIndex + 1) * (yAxis->maxIndex
-                                                 - yAxis->minIndex + 1);
+    zoffset = pow10(power10z) * zAxis->bias;
+	
+    dfield = gwy_data_field_new(nx,ny, xreal, yreal, FALSE);
+    total = nx * ny;
     gwy_data_field_set_si_unit_xy(dfield, siunitxy);
     g_object_unref(siunitxy);
     gwy_data_field_set_si_unit_z(dfield, siunitz);
@@ -1777,7 +1776,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const gchar *tp = p;
 
             while (data < end_data)
-                *(data++) = zscale * (*(tp++));
+                *(data++) = zoffset + zscale * (*(tp++));
         }
         break;
 
@@ -1786,7 +1785,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const guchar *tp = (const guchar *)p;
 
             while (data < end_data)
-                *(data++) = zscale * (*(tp++));
+                *(data++) = zoffset + zscale * (*(tp++));
         }
         break;
 
@@ -1795,7 +1794,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const gint16 *tp = (const gint16 *)p;
 
             while (data < end_data) {
-                *(data++) = zscale * GINT16_FROM_LE(*tp);
+                *(data++) = zoffset + zscale * GINT16_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1806,7 +1805,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const guint16 *tp = (const guint16 *)p;
 
             while (data < end_data) {
-                *(data++) = zscale * GUINT16_FROM_LE(*tp);
+                *(data++) = zoffset + zscale * GUINT16_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1817,7 +1816,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const gint32 *tp = (const gint32 *)p;
 
             while (data < end_data) {
-                *(data++) = zscale * GINT32_FROM_LE(*tp);
+                *(data++) = zoffset + zscale * GINT32_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1828,7 +1827,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const guint32 *tp = (const guint32 *)p;
 
             while (data < end_data) {
-                *(data++) = zscale * GUINT32_FROM_LE(*tp);
+                *(data++) = zoffset + zscale * GUINT32_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1839,9 +1838,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const gint64 *tp = (const gint64 *)p;
 
             while (data < end_data) {
-                /* for some reason, MSVC6 spits an unsigned int64 conversion
-                 * error also here */
-                *(data++) = zscale * (gint64)GINT64_FROM_LE(*tp);
+                *(data++) = zoffset + zscale * (gint64)GINT64_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1852,15 +1849,7 @@ extract_mda_data(MDTMDAFrame * dataframe)
             const guint64 *tp = (const guint64 *)p;
 
             while (data < end_data) {
-                /* Fucking MSVC6 cannot convert unsigned 64bit int to double. */
-#ifdef _MSC_VER
-                guint u32h = *tp >> 32u;
-                guint u32l = *tp & 0xffffffffu;
-                *(data++) = zscale * (4294967296.0*u32h + u32l);
-
-#else
-                *(data++) = zscale * GUINT64_FROM_LE(*tp);
-#endif
+                *(data++) = zoffset + zscale * GUINT64_FROM_LE(*tp);
                 tp++;
             }
         }
@@ -1868,12 +1857,12 @@ extract_mda_data(MDTMDAFrame * dataframe)
 
         case MDA_DATA_FLOAT32:
         while (data < end_data)
-            *(data++) = zscale * gwy_get_gfloat_le(&p);
+            *(data++) = zoffset + zscale * gwy_get_gfloat_le(&p);
         break;
 
         case MDA_DATA_FLOAT64:
         while (data < end_data)
-            *(data++) = zscale * gwy_get_gdouble_le(&p);
+            *(data++) = zoffset + zscale * gwy_get_gdouble_le(&p);
         break;
 
         default:
@@ -2028,8 +2017,6 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
                 {
                     const gint64 *tp = (const gint64 *)p;
 
-                    /* for some reason, MSVC6 spits an unsigned int64 conversion
-                     * error also here */
                     xdata[i] = xscale * (gint64)GINT64_FROM_LE(*tp);
                     p += sizeof(gint64)/sizeof(gchar);
                 }
@@ -2038,15 +2025,7 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
                 case MDA_DATA_UINT64:
                 {
                     const guint64 *tp = (const guint64 *)p;
-
-                    /* Fucking MSVC6 cannot convert unsigned 64bit int to double. */
-#ifdef _MSC_VER
-                    guint u32h = *tp >> 32u;
-                    guint u32l = *tp & 0xffffffffu;
-                    xdata[i] = xscale * (4294967296.0*u32h + u32l);
-#else
                     xdata[i] = xscale * GUINT64_FROM_LE(*tp);
-#endif
                     p += sizeof(guint64)/sizeof(gchar);
                 }
                 break;
@@ -2121,8 +2100,6 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
             {
                 const gint64 *tp = (const gint64 *)p;
 
-                /* for some reason, MSVC6 spits an unsigned int64 conversion
-                 * error also here */
                 ydata[i] = yscale * (gint64)GINT64_FROM_LE(*tp);
                 p += sizeof(gint64);
             }
@@ -2131,15 +2108,7 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
             case MDA_DATA_UINT64:
             {
                 const guint64 *tp = (const guint64 *)p;
-
-                /* Fucking MSVC6 cannot convert unsigned 64bit int to double. */
-#ifdef _MSC_VER
-                guint u32h = *tp >> 32u;
-                guint u32l = *tp & 0xffffffffu;
-                ydata[i] = yscale * (4294967296.0*u32h + u32l);
-#else
                 ydata[i] = yscale * GUINT64_FROM_LE(*tp);
-#endif
                 p += sizeof(guint64);
             }
             break;
@@ -2391,7 +2360,7 @@ static GwyDataField * extract_raman_image (MDTMDAFrame *dataframe,
 
     /* FIXME: real size is wrong */
     dfield = gwy_data_field_new(xsize, ysize,
-                                xreal*xsize, yreal*ysize, FALSE);
+                                xreal*(xsize - 1), yreal*(ysize - 1), FALSE);
 
     gwy_data_field_set_si_unit_xy(dfield, siunitxy);
     g_object_unref(siunitxy);
