@@ -1,5 +1,5 @@
 /*
- *  $Id: opdfile.c 10195 2009-10-01 23:04:26Z yeti-dn $
+ *  $Id: opdfile.c 12231 2011-03-20 22:32:00Z yeti-dn $
  *  Copyright (C) 2008 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -179,7 +179,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Wyko OPD and ASC files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.6",
+    "0.7",
     "David Nečas (Yeti)",
     "2008",
 };
@@ -318,7 +318,8 @@ opd_load(const gchar *filename,
 
         if (!gwy_stramong(header[i].name,
                           "OPD", "SAMPLE_DATA", "RAW_DATA", "RAW DATA",
-                          "Image", "Intensity", "SecArr_0", NULL))
+                          "Image", "Intensity", "SecArr_0", "Raw",
+                          NULL))
             continue;
 
         if (header[i].type != OPD_ARRAY) {
@@ -378,7 +379,7 @@ opd_load(const gchar *filename,
         g_object_unref(meta);
     }
     else {
-        g_object_unref(container);
+        gwy_object_unref(container);
         err_NO_DATA(error);
     }
 
@@ -733,7 +734,8 @@ opd_asc_load(const gchar *filename,
         s++;
 
         if (gwy_stramong(line, "OPD", "SAMPLE_DATA", "RAW_DATA", "RAW DATA",
-                         "Image", "Intensity", "SecArr_0", NULL)) {
+                         "Image", "Intensity", "SecArr_0", "Raw",
+                         NULL)) {
             gdouble pixel_size, wavelength, mult = 1.0, aspect = 1.0;
             gboolean intensity;
             gchar *k;
