@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: psia.c 9232 2009-06-09 14:29:04Z yeti-dn $
+ *  @(#) $Id: psia.c 10431 2009-11-14 07:36:16Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -18,10 +18,16 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * PSIA
+ * .tiff
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdio.h>
-#include <stdarg.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwymath.h>
 #include <libgwyddion/gwyutils.h>
@@ -494,7 +500,7 @@ psia_read_image_header(const guchar *p,
 {
     guint bps = 0;
 
-    memset(header, 0, sizeof(PSIAImageHeader));
+    gwy_clear(header, 1);
 
     if ((version == PSIA_VERSION1 && size < 356)
         || (version == PSIA_VERSION2 && size < 580)) {
@@ -505,7 +511,7 @@ psia_read_image_header(const guchar *p,
     }
 
     /* Parse header */
-    memset(header, 0, sizeof(PSIAImageHeader));
+    gwy_clear(header, 1);
     header->image_type = gwy_get_guint32_le(&p);
     gwy_debug("image_type: %d", header->image_type);
     if (header->image_type != PSIA_2D_MAPPED
@@ -608,7 +614,7 @@ psia_read_spectro_header(const guchar *p,
 {
     guint i;
 
-    memset(header, 0, sizeof(PSIASpectroscopyHeader));
+    gwy_clear(header, 1);
 
     if (size < 536) {
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
