@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: rawfile.c 10237 2009-10-13 09:33:28Z yeti-dn $
+ *  @(#) $Id: rawfile.c 12243 2011-03-27 14:15:54Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -781,7 +781,7 @@ rawfile_dialog_format_page(RawFileArgs *args,
                                    -1,
                                    _("Any whitespace"), RAW_DELIM_WHITESPACE,
                                    _("TAB character"), RAW_DELIM_TAB,
-                                   _("Ohter character"), RAW_DELIM_OTHER,
+                                   _("Other character"), RAW_DELIM_OTHER,
                                    NULL);
     controls->delimmenu = combo;
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
@@ -2057,16 +2057,7 @@ rawfile_read_builtin(RawFileArgs *args,
                 break;
 
                 case RAW_UNSIGNED_WORD64:
-                /* Fucking MSVC6 cannot convert unsigned 64bit int to double. */
-#ifdef _MSC_VER
-                {
-                    guint u32h = good_alignment.u64 >> 32u;
-                    guint u32l = good_alignment.u64 & 0xffffffffu;
-                    *(data++) = 4294967296.0*u32h + u32l;
-                }
-#else
                 *(data++) = (gdouble)good_alignment.u64;
-#endif
                 break;
 
                 default:
