@@ -1,5 +1,5 @@
 /*
- *  $Id: rawxyz.c 10344 2009-11-02 22:31:36Z yeti-dn $
+ *  $Id: rawxyz.c 11602 2010-11-23 09:50:28Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -1024,7 +1024,7 @@ rawxyz_do(RawXYZFile *rfile,
         fill_field_x((const GwyTriangulationPointXYZ*)points->data, dfield);
     else if (rfile->regular == RAW_XYZ_REGULAR_Y)
         fill_field_y((const GwyTriangulationPointXYZ*)points->data, dfield);
-    else if (args->interpolation == GWY_INTERPOLATION_FIELD) {
+    else if ((gint)args->interpolation == GWY_INTERPOLATION_FIELD) {
         extend_borders(rfile, args, EPSREL);
         interpolate_field(points->len,
                           (const GwyTriangulationPointXYZ*)points->data,
@@ -1709,7 +1709,7 @@ static void
 rawxyz_sanitize_args(RawXYZArgs *args)
 {
     if (args->interpolation != GWY_INTERPOLATION_ROUND
-        && args->interpolation != GWY_INTERPOLATION_FIELD)
+        && (gint)args->interpolation != GWY_INTERPOLATION_FIELD)
         args->interpolation = GWY_INTERPOLATION_LINEAR;
     if (args->exterior != GWY_EXTERIOR_MIRROR_EXTEND
         && args->exterior != GWY_EXTERIOR_PERIODIC)
