@@ -1,5 +1,5 @@
 /*
- *  $Id: stpfile.c 10195 2009-10-01 23:04:26Z yeti-dn $
+ *  $Id: stpfile.c 12269 2011-04-07 09:21:30Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klstptek.
  *  E-mail: yeti@gwyddion.net, klstptek@gwyddion.net.
  *
@@ -524,6 +524,4 @@ read_data_field(GwyDataField *dfield,
     }
 }
 
-/* vim: set cin et ts=4 sw=4
-cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
-
+/* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
