@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: axis.c 7511 2007-02-11 18:49:21Z yeti-dn $
+ *  @(#) $Id: axis.c 9259 2009-06-16 09:35:11Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -90,6 +90,14 @@ static void     gwy_selection_axis_get_property   (GObject*object,
                                                    guint prop_id,
                                                    GValue *value,
                                                    GParamSpec *pspec);
+static gboolean gwy_selection_axis_crop_object    (GwySelection *selection,
+                                                   gint i,
+                                                   gpointer user_data);
+static void     gwy_selection_axis_crop           (GwySelection *selection,
+                                                   gdouble xmin,
+                                                   gdouble ymin,
+                                                   gdouble xmax,
+                                                   gdouble ymax);
 static GByteArray* gwy_selection_axis_serialize   (GObject *serializable,
                                                    GByteArray *buffer);
 static GObject* gwy_selection_axis_deserialize    (const guchar *buffer,
@@ -130,7 +138,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Layer allowing selection of horizontal or vertical lines."),
     "Yeti <yeti@gwyddion.net>",
-    "2.5",
+    "2.6",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -159,6 +167,7 @@ gwy_selection_axis_class_init(GwySelectionAxisClass *klass)
     gobject_class->get_property = gwy_selection_axis_get_property;
 
     sel_class->object_size = OBJECT_SIZE;
+    sel_class->crop = gwy_selection_axis_crop;
 
     g_object_class_install_property
         (gobject_class,
@@ -234,7 +243,7 @@ gwy_selection_axis_set_property(GObject *object,
 }
 
 static void
-gwy_selection_axis_get_property(GObject*object,
+gwy_selection_axis_get_property(GObject *object,
                                 guint prop_id,
                                 GValue *value,
                                 GParamSpec *pspec)
@@ -252,6 +261,34 @@ gwy_selection_axis_get_property(GObject*object,
     }
 }
 
+static gboolean
+gwy_selection_axis_crop_object(GwySelection *selection,
+                               gint i,
+                               gpointer user_data)
+{
+    const gdouble *minmax = (const gdouble*)user_data;
+    GwySelectionAxis *selection_axis = GWY_SELECTION_AXIS(selection);
+    gdouble xy[OBJECT_SIZE];
+
+    gwy_selection_get_object(selection, i, xy);
+    if (selection_axis->orientation == GWY_ORIENTATION_VERTICAL)
+        return xy[0] >= minmax[1] && xy[0] <= minmax[3];
+    else
+        return xy[0] >= minmax[0] && xy[0] <= minmax[2];
+}
+
+static void
+gwy_selection_axis_crop(GwySelection *selection,
+                        gdouble xmin,
+                        gdouble ymin,
+                        gdouble xmax,
+                        gdouble ymax)
+{
+    gdouble minmax[4] = { xmin, ymin, xmax, ymax };
+
+    gwy_selection_filter(selection, gwy_selection_axis_crop_object, minmax);
+}
+
 static GByteArray*
 gwy_selection_axis_serialize(GObject *serializable,
                              GByteArray *buffer)
@@ -336,9 +373,10 @@ static void
 gwy_selection_axis_clone(GObject *source,
                          GObject *copy)
 {
-    gwy_selection_axis_serializable_parent_iface->clone(source, copy);
     GWY_SELECTION_AXIS(copy)->orientation
         = GWY_SELECTION_AXIS(source)->orientation;
+    /* Must do this at the end, it emits a signal. */
+    gwy_selection_axis_serializable_parent_iface->clone(source, copy);
 }
 
 static void
