@@ -1,5 +1,5 @@
 /*
- *  @(%) $Id: plugin-proxy.c 11560 2010-11-15 13:37:12Z yeti-dn $
+ *  @(%) $Id: plugin-proxy.c 11681 2010-12-10 07:56:23Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -46,20 +46,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
-
-#if (defined(HAVE_SYS_STAT_H) || defined(_WIN32))
 #include <sys/stat.h>
-/* And now we are in a deep s... */
-#endif
-
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-
 #include <glib/gstdio.h>
 #include <libgwyddion/gwymacros.h>
 #include <libgwymodule/gwymodule.h>
