@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: arithmetic.c 11539 2010-11-12 14:18:50Z yeti-dn $
+ *  @(#) $Id: arithmetic.c 11744 2011-01-11 08:26:44Z yeti-dn $
  *  Copyright (C) 2003-2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -639,7 +639,7 @@ arithmetic_do(ArithmeticArgs *args)
 
     /* Then the mask fields */
     for (i = NARGS*ARITHMETIC_MASK; i < NARGS*(ARITHMETIC_MASK + 1); i++) {
-        gwy_debug("mfield[%u]: %s", i, need_data[i] ? "NEEDED" : "not needed");
+        gwy_debug("mfield[%u]: %s", i, need_data[i % NARGS] ? "NEEDED" : "not needed");
         if (!need_data[i % NARGS])
             continue;
 
