@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: calibrate.c 9121 2009-04-06 15:46:38Z yeti-dn $
+ *  @(#) $Id: calibrate.c 11854 2011-01-25 07:11:03Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -156,11 +156,12 @@ module_register(void)
 {
     gwy_process_func_register("calibrate",
                               (GwyProcessFunc)&calibrate,
-                              N_("/_Basic Operations/_Recalibrate..."),
+                              N_("/_Basic Operations/_Dimensions and Units..."),
                               GWY_STOCK_DATA_MEASURE,
                               CALIBRATE_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
-                              N_("Recalibrate data"));
+                              N_("Change physical dimensions, units "
+                                 "or value scale"));
 
     return TRUE;
 }
@@ -291,7 +292,7 @@ calibrate_dialog(CalibrateArgs *args,
     CalibrateControls controls;
     gint row, response;
 
-    dialog = gtk_dialog_new_with_buttons(_("Calibrate"), NULL, 0,
+    dialog = gtk_dialog_new_with_buttons(_("Dimensions and Units"), NULL, 0,
                                          _("_Reset"), RESPONSE_RESET,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
