@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: curvature.c 11576 2010-11-16 14:58:10Z yeti-dn $
+ *  @(#) $Id: curvature.c 12247 2011-03-28 14:48:18Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.net.
  *
@@ -653,7 +653,7 @@ curvature_dialog(CurvatureArgs *args,
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
     row = 0;
 
-    label = gwy_label_new_header(_("Output type"));
+    label = gwy_label_new_header(_("Output Type"));
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label,
                      0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
