@@ -739,9 +739,11 @@ page_switched(DepositSynthControls *controls,
     if (pagenum == PAGE_GENERATOR) {
         GwyDimensions *dims = controls->dims;
 
-        gtk_label_set_markup(GTK_LABEL(controls->size_units),
+        if (controls->size_units)
+          gtk_label_set_markup(GTK_LABEL(controls->size_units),
                              dims->xyvf->units);
-        gtk_label_set_markup(GTK_LABEL(controls->width_units),
+        if (controls->width_units)
+          gtk_label_set_markup(GTK_LABEL(controls->width_units),
                              dims->xyvf->units);
 
         //update_size_value(controls);
@@ -1212,15 +1214,16 @@ deposit_synth_do(const DepositSynthArgs *args,
                 ii++;
                 too_close = 0;
 
+                rdisizes[ndata] = size;
                 rxv = ((gdouble)xpos*oxreal/(gdouble)oxres); 
                 ryv = ((gdouble)ypos*oyreal/(gdouble)oyres);
-                rzv = gwy_data_field_get_val(zlfield, xpos, ypos) + 5*size;
+                rzv = gwy_data_field_get_val(zlfield, xpos, ypos)+ rdisizes[ndata];// + 5*size; 
 
                 for (k=0; k<ndata; k++)
                 {
                     if (((rxv-rx[k])*(rxv-rx[k])
                          + (ryv-ry[k])*(ryv-ry[k])
-                         + (rzv-rz[k])*(rzv-rz[k]))<(4.0*size*size))
+                         + (rzv-rz[k])*(rzv-rz[k]))<(4.0*size*size)) 
                     {
                         too_close = 1;
                         break;
@@ -1238,7 +1241,7 @@ deposit_synth_do(const DepositSynthArgs *args,
                 rx[ndata] = rxv;
                 ry[ndata] = ryv;
                 rz[ndata] = rzv;
-                vz[ndata] = -0.005;
+                vz[ndata] = 0;//-0.005;
                 ndata++;
                 nloc++;
 
