@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: fft_profile.c 9300 2009-06-22 07:09:40Z yeti-dn $
+ *  @(#) $Id: fft_profile.c 11330 2010-06-24 08:13:31Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -131,7 +131,7 @@ module_register(void)
 {
     gwy_process_func_register("fft_profile",
                               (GwyProcessFunc)&fft_profile,
-                              N_("/_Statistics/_PDSF Section..."),
+                              N_("/_Statistics/_PSDF Section..."),
                               GWY_STOCK_GRAPH_HALFGAUSS,
                               PROF_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
