@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: indent_analyze.c 9081 2009-03-17 12:15:42Z yeti-dn $
+ *  @(#) $Id: indent_analyze.c 12243 2011-03-27 14:15:54Z yeti-dn $
  *  Copyright (C) 2006 Lukas Chvatal, David Necas (Yeti), Petr Klapetek.
  *  E-mail: chvatal@physics.muni.cz, yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -491,7 +491,7 @@ create_parameters_table(IndentAnalyzeControls * controls)
    gwy_table_attach_row(table, row++, _("Maximum at"), siu->str,
          controls->w_max_xy);
    controls->w_minmax = gtk_label_new("");
-   gwy_table_attach_row(table, row++, _("Max-min diference"), siu->str,
+   gwy_table_attach_row(table, row++, _("Max-min difference"), siu->str,
          controls->w_minmax);
 
    g_string_append(siu, "<sup>2</sup>");
@@ -648,7 +648,7 @@ indent_analyze_dialog(GwyContainer *data, IndentAnalyzeArgs * args)
     GtkObject *layer;
 
     controls.args = args;
-    dialog = gtk_dialog_new_with_buttons(_("Indentaion statistics"), NULL, 0,
+    dialog = gtk_dialog_new_with_buttons(_("Indentation statistics"), NULL, 0,
                                          _("_Compute & mark"), RESPONSE_COMPUTE,
                                          _("_Save statistics"), RESPONSE_SAVE,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -1578,7 +1578,7 @@ save_statistics_dialog(IndentAnalyzeControls *c,
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_OK,
-                                        _("No statistics has benn "
+                                        _("No statistics has been "
                                           "computed yet."));
         gtk_dialog_run(GTK_DIALOG(dialog));
         gtk_widget_destroy(dialog);
@@ -1634,7 +1634,7 @@ save_statistics_dialog(IndentAnalyzeControls *c,
                 args->minx, args->miny, args->min_val/mag);
         fprintf(out, _("Maximum at [%d, %d] is:                 %lf\n"),
                 args->maxx, args->maxy, args->max_val/mag);
-        fprintf(out, _("Diference max-min:                      %lf\n"),
+        fprintf(out, _("Difference max-min:                     %lf\n"),
                 (args->max_val - args->min_val)/mag);
         fprintf(out, "\n");
 
@@ -1663,7 +1663,7 @@ save_statistics_dialog(IndentAnalyzeControls *c,
                 args->volume_above/mag/mag/mag);
         fprintf(out, _("Volume below:     %g\n"),
                 args->volume_below/mag/mag/mag);
-        fprintf(out, _("Volume diference  %g\n"),
+        fprintf(out, _("Volume difference %g\n"),
                 (args->volume_above -
                  args->volume_below)/mag/mag/mag);
 
