@@ -1,4 +1,4 @@
-/*  @(#) $Id: nanoindent_adjust.c 6991 2006-11-16 22:11:37Z yeti-dn $
+/*  @(#) $Id: nanoindent_adjust.c 9032 2009-03-05 15:29:26Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -319,7 +319,7 @@ nanoindent_adjust_do(NanoindentAdjustArgs *args)
 
     dfield = gwy_data_field_duplicate(dfield1);
 
-    gwy_app_wait_start(GTK_WIDGET(args->win2), "Initializing...");
+    gwy_app_wait_start(GTK_WIDGET(args->win2), _("Initializing..."));
     dfield = gwy_nanoindent_adjust(dfield1, dfield,
                               gwy_app_wait_set_fraction,
                               gwy_app_wait_set_message,
