@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: polydistort.c 8912 2008-12-30 12:42:18Z yeti-dn $
+ *  @(#) $Id: polydistort.c 10348 2009-11-02 22:56:33Z yeti-dn $
  *  Copyright (C) 2007,2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -111,7 +111,7 @@ static void       distort_dialog_update_controls(DistortControls *controls,
 static void       distort_coeff_changed         (GtkEntry *entry,
                                                  DistortControls *controls);
 static void       distort_invalidate            (DistortControls *controls);
-static void       preview_type_changed          (GtkWidget *button,
+static void       preview_type_changed          (GtkToggleButton *button,
                                                  DistortControls *controls);
 static void       preview                       (DistortControls *controls,
                                                  DistortArgs *args);
@@ -278,7 +278,7 @@ distort_dialog(DistortArgs *args,
                                   _("Mirror"), GWY_EXTERIOR_MIRROR_EXTEND,
                                   _("Periodic"), GWY_EXTERIOR_PERIODIC,
                                   NULL);
-    gwy_table_attach_hscale(table, row, _("_Exteriror type:"), NULL,
+    gwy_table_attach_hscale(table, row, _("_Exterior type:"), NULL,
                             GTK_OBJECT(controls.exterior),
                             GWY_HSCALE_WIDGET_NO_EXPAND);
     gtk_table_set_row_spacing(GTK_TABLE(table), row, 8);
@@ -619,11 +619,14 @@ distort_invalidate(DistortControls *controls)
 }
 
 static void
-preview_type_changed(G_GNUC_UNUSED GtkWidget *button,
+preview_type_changed(GtkToggleButton *button,
                      DistortControls *controls)
 {
     GwyPixmapLayer *blayer;
 
+    if (button && !gtk_toggle_button_get_active(button))
+        return;
+
     controls->args->preview_type
         = gwy_radio_buttons_get_current(controls->preview_type);
 
@@ -815,4 +818,3 @@ distort_save_args(GwyContainer *container,
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
-
