@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: slope_dist.c 10297 2009-10-26 19:27:45Z yeti-dn $
+ *  @(#) $Id: slope_dist.c 12247 2011-03-28 14:48:18Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -213,7 +213,7 @@ slope_dialog(SlopeArgs *args, gboolean same_units)
                        FALSE, FALSE, 4);
     row = 0;
 
-    label = gtk_label_new(_("Output type"));
+    label = gtk_label_new(_("Output type:"));
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label,
                      0, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
