@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: stats.c 10205 2009-10-05 08:31:16Z yeti-dn $
+ *  @(#) $Id: stats.c 12188 2011-03-04 12:44:58Z acampbellova $
  *  Copyright (C) 2003-2008 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -29,6 +29,7 @@
 #include <libgwymodule/gwymodule-tool.h>
 #include <libprocess/datafield.h>
 #include <libprocess/stats.h>
+#include <libprocess/stats_uncertainty.h>
 #include <libgwydgets/gwystock.h>
 #include <libgwydgets/gwyradiobuttons.h>
 #include <libgwydgets/gwydgetutils.h>
@@ -65,6 +66,19 @@ typedef struct {
     /* These two are in degrees as we use need only for the user. */
     gdouble theta;
     gdouble phi;
+
+    gdouble uavg;
+    gdouble umin;
+    gdouble umax;
+    gdouble umedian;
+    gdouble ura;
+    gdouble urms;
+    gdouble uskew;
+    gdouble ukurtosis;
+    gdouble uarea;
+    gdouble uprojarea;
+    gdouble utheta;
+    gdouble uphi;
 } ToolResults;
 
 typedef struct {
@@ -110,6 +124,12 @@ struct _GwyToolStats {
 
     gboolean same_units;
 
+    gboolean has_calibration;
+    GwyDataField *xunc;
+    GwyDataField *yunc;
+    GwyDataField *zunc;
+
+
     /* potential class data */
     GwySIValueFormat *angle_format;
     GType layer_type_rect;
@@ -141,6 +161,10 @@ static void  gwy_tool_stats_update_units          (GwyToolStats *tool);
 static void  update_label                         (GwySIValueFormat *units,
                                                    GtkWidget *label,
                                                    gdouble value);
+static void  update_label_unc                     (GwySIValueFormat *units,
+                                                   GtkWidget *label,
+                                                   gdouble value,
+                                                   gdouble uncertainty);
 static void  gwy_tool_stats_masking_changed       (GtkWidget *button,
                                                    GwyToolStats *tool);
 static void  gwy_tool_stats_instant_update_changed(GtkToggleButton *check,
@@ -298,8 +322,8 @@ gwy_tool_stats_init_dialog(GwyToolStats *tool)
         { N_("Minimum:"),           G_STRUCT_OFFSET(GwyToolStats, min),      },
         { N_("Maximum:"),           G_STRUCT_OFFSET(GwyToolStats, max),      },
         { N_("Median:"),            G_STRUCT_OFFSET(GwyToolStats, median),   },
-        { N_("Ra:"),                G_STRUCT_OFFSET(GwyToolStats, ra),       },
-        { N_("Rms:"),               G_STRUCT_OFFSET(GwyToolStats, rms),      },
+        { N_("Ra (Sa):"),            G_STRUCT_OFFSET(GwyToolStats, ra),       },
+        { N_("Rms (Sq):"),           G_STRUCT_OFFSET(GwyToolStats, rms),      },
         { N_("Skew:"),              G_STRUCT_OFFSET(GwyToolStats, skew),     },
         { N_("Kurtosis:"),          G_STRUCT_OFFSET(GwyToolStats, kurtosis), },
         { N_("Surface area:"),      G_STRUCT_OFFSET(GwyToolStats, area),     },
@@ -424,6 +448,10 @@ gwy_tool_stats_data_switched(GwyTool *gwytool,
     GwyPlainTool *plain_tool;
     GwyToolStats *tool;
     gboolean ignore;
+    gchar xukey[24];
+    gchar yukey[24];
+    gchar zukey[24];
+
 
     plain_tool = GWY_PLAIN_TOOL(gwytool);
     tool = GWY_TOOL_STATS(gwytool);
@@ -446,10 +474,25 @@ gwy_tool_stats_data_switched(GwyTool *gwytool,
                                 "focus", -1,
                                 NULL);
         gwy_selection_set_max_objects(plain_tool->selection, 1);
+
+        g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
+        g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
+        g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
+
+        if (gwy_container_gis_object_by_name(plain_tool->container, xukey, &(tool->xunc))
+            && gwy_container_gis_object_by_name(plain_tool->container, yukey, &(tool->yunc))
+            && gwy_container_gis_object_by_name(plain_tool->container, zukey, &(tool->zunc)))
+        {
+            tool->has_calibration = TRUE;
+        } else {
+            tool->has_calibration = FALSE;
+        }
+        gwy_tool_stats_update_labels(tool);
     }
 
     gtk_widget_set_sensitive(tool->copy, data_view != NULL);
     gtk_widget_set_sensitive(tool->save, data_view != NULL);
+
 }
 
 static void
@@ -480,6 +523,22 @@ gwy_tool_stats_update_units(GwyToolStats *tool)
 static void
 gwy_tool_stats_data_changed(GwyPlainTool *plain_tool)
 {
+    gchar xukey[24];
+    gchar yukey[24];
+    gchar zukey[24];
+    
+    g_snprintf(xukey, sizeof(xukey), "/%d/data/cal_xunc", plain_tool->id);
+    g_snprintf(yukey, sizeof(yukey), "/%d/data/cal_yunc", plain_tool->id);
+    g_snprintf(zukey, sizeof(zukey), "/%d/data/cal_zunc", plain_tool->id);
+
+    if (gwy_container_gis_object_by_name(plain_tool->container, xukey, &(GWY_TOOL_STATS(plain_tool)->xunc))
+        && gwy_container_gis_object_by_name(plain_tool->container, yukey, &(GWY_TOOL_STATS(plain_tool)->yunc))
+        && gwy_container_gis_object_by_name(plain_tool->container, zukey, &(GWY_TOOL_STATS(plain_tool)->zunc)))
+    {
+        GWY_TOOL_STATS(plain_tool)->has_calibration = TRUE;
+    } else {
+        GWY_TOOL_STATS(plain_tool)->has_calibration = FALSE;
+    }
 
     gwy_rect_selection_labels_fill(GWY_TOOL_STATS(plain_tool)->rlabels,
                                    plain_tool->selection,
@@ -563,27 +622,51 @@ gwy_tool_stats_update_labels(GwyToolStats *tool)
     if (!tool->results_up_to_date)
         return;
 
-    update_label(plain_tool->value_format, tool->ra, tool->results.ra);
-    update_label(plain_tool->value_format, tool->rms, tool->results.rms);
-    g_snprintf(buffer, sizeof(buffer), "%2.3g", tool->results.skew);
-    gtk_label_set_text(GTK_LABEL(tool->skew), buffer);
-    g_snprintf(buffer, sizeof(buffer), "%2.3g", tool->results.kurtosis);
-    gtk_label_set_text(GTK_LABEL(tool->kurtosis), buffer);
-    update_label(plain_tool->value_format, tool->avg, tool->results.avg);
-    update_label(plain_tool->value_format, tool->min, tool->results.min);
-    update_label(plain_tool->value_format, tool->max, tool->results.max);
-    update_label(plain_tool->value_format, tool->median, tool->results.median);
+    if (tool->has_calibration) {
+        update_label_unc(plain_tool->value_format, tool->ra, tool->results.ra, tool->results.ura);
+        update_label_unc(plain_tool->value_format, tool->rms, tool->results.rms, tool->results.urms);
+        g_snprintf(buffer, sizeof(buffer), "%2.3g±%2.3g", tool->results.skew, tool->results.uskew);
+        gtk_label_set_text(GTK_LABEL(tool->skew), buffer);
+        g_snprintf(buffer, sizeof(buffer), "%2.3g±%2.3g", tool->results.kurtosis, tool->results.ukurtosis);
+        gtk_label_set_text(GTK_LABEL(tool->kurtosis), buffer);
+        update_label_unc(plain_tool->value_format, tool->avg, tool->results.avg, tool->results.uavg);
+        update_label_unc(plain_tool->value_format, tool->min, tool->results.min, tool->results.umin);
+        update_label_unc(plain_tool->value_format, tool->max, tool->results.max, tool->results.umax);
+        update_label_unc(plain_tool->value_format, tool->median, tool->results.median, tool->results.umedian);
+        update_label_unc(tool->area_format, tool->projarea, tool->results.projarea, tool->results.uprojarea);
+//        update_label(tool->area_format, tool->projarea, tool->results.projarea);
+    } else {
+        update_label(plain_tool->value_format, tool->ra, tool->results.ra);
+        update_label(plain_tool->value_format, tool->rms, tool->results.rms);
+        g_snprintf(buffer, sizeof(buffer), "%2.3g", tool->results.skew);
+        gtk_label_set_text(GTK_LABEL(tool->skew), buffer);
+        g_snprintf(buffer, sizeof(buffer), "%2.3g", tool->results.kurtosis);
+        gtk_label_set_text(GTK_LABEL(tool->kurtosis), buffer);
+        update_label(plain_tool->value_format, tool->avg, tool->results.avg);
+        update_label(plain_tool->value_format, tool->min, tool->results.min);
+        update_label(plain_tool->value_format, tool->max, tool->results.max);
+        update_label(plain_tool->value_format, tool->median, tool->results.median);
 
-    update_label(tool->area_format, tool->projarea, tool->results.projarea);
+        update_label(tool->area_format, tool->projarea, tool->results.projarea);
+    }
 
     if (tool->same_units)
-        update_label(tool->area_format, tool->area, tool->results.area);
+    {
+        if (!tool->has_calibration) 
+           update_label(tool->area_format, tool->area, tool->results.area);
+        else
+           update_label_unc(tool->area_format, tool->area, tool->results.area, tool->results.uarea);
+    }
     else
         gtk_label_set_text(GTK_LABEL(tool->area), _("N.A."));
 
     if (tool->same_units && !mask_in_use) {
         update_label(tool->angle_format, tool->theta, tool->results.theta);
         update_label(tool->angle_format, tool->phi, tool->results.phi);
+        if (tool->has_calibration) {
+		    update_label_unc(tool->angle_format, tool->theta, tool->results.theta, tool->results.utheta);
+		    update_label_unc(tool->angle_format, tool->phi, tool->results.phi, tool->results.uphi);
+	}
     }
     else {
         gtk_label_set_text(GTK_LABEL(tool->theta), _("N.A."));
@@ -600,6 +683,7 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
     gdouble q;
     gint nn, w, h;
     gdouble sel[4];
+    gint oldx, oldy;
     gint isel[4];
 
     plain_tool = GWY_PLAIN_TOOL(tool);
@@ -683,6 +767,74 @@ gwy_tool_stats_calculate(GwyToolStats *tool)
         tool->results.theta *= 180.0/G_PI;
         tool->results.phi *= 180.0/G_PI;
     }
+    if (tool->has_calibration) {
+
+        oldx = gwy_data_field_get_xres(tool->xunc);
+        oldy = gwy_data_field_get_yres(tool->xunc);
+        gwy_data_field_resample(tool->xunc,                     //FIXME, functions should work with data of any size
+                                gwy_data_field_get_xres(plain_tool->data_field), 
+                                gwy_data_field_get_yres(plain_tool->data_field),
+                                GWY_INTERPOLATION_BILINEAR);
+        gwy_data_field_resample(tool->yunc, 
+                                gwy_data_field_get_xres(plain_tool->data_field), 
+                                gwy_data_field_get_yres(plain_tool->data_field),
+                                GWY_INTERPOLATION_BILINEAR);
+        gwy_data_field_resample(tool->zunc, 
+                                gwy_data_field_get_xres(plain_tool->data_field), 
+                                gwy_data_field_get_yres(plain_tool->data_field),
+                                GWY_INTERPOLATION_BILINEAR);
+
+
+
+        tool->results.uprojarea = gwy_data_field_area_get_projected_area_uncertainty(nn, tool->xunc, tool->yunc);
+        
+
+        gwy_data_field_area_get_stats_uncertainties_mask(plain_tool->data_field, tool->zunc, mask, masking,
+                                           isel[0], isel[1], w, h,
+                                           &tool->results.uavg,
+                                           &tool->results.ura,
+                                           &tool->results.urms,
+                                           &tool->results.uskew,
+                                           &tool->results.ukurtosis);
+
+        gwy_data_field_area_get_min_max_uncertainty_mask(plain_tool->data_field, tool->zunc, mask, masking,
+                                             isel[0], isel[1], w, h,
+                                             &tool->results.umin,
+                                             &tool->results.umax);
+        tool->results.umedian
+            = gwy_data_field_area_get_median_uncertainty_mask(plain_tool->data_field, tool->zunc,
+                                                  mask, masking,
+                                                  isel[0], isel[1], w, h);
+        tool->results.uarea = 0;
+         /*   = gwy_data_field_area_get_surface_area_mask_uncertainty(plain_tool->data_field, tool->zunc,
+                                                               tool->xunc,
+                                                               tool->yunc,
+                                                               mask,masking,
+                                                               isel[0], isel[1], w, h);
+           TODO							       */
+	if (tool->same_units && !mask) {
+		gwy_data_field_area_get_inclination_uncertainty(plain_tool->data_field,
+				tool->zunc, tool->xunc, tool->yunc,
+				isel[0], isel[1], w, h,
+				&tool->results.utheta,
+				&tool->results.uphi);
+		tool->results.utheta *= 180.0/G_PI;
+		tool->results.uphi *= 180.0/G_PI;
+	}
+
+        gwy_data_field_resample(tool->xunc, 
+                                oldx, 
+                                oldy,
+                                GWY_INTERPOLATION_BILINEAR);
+        gwy_data_field_resample(tool->yunc, 
+                                oldx, 
+                                oldy,
+                                GWY_INTERPOLATION_BILINEAR);
+        gwy_data_field_resample(tool->zunc, 
+                                oldx, 
+                                oldy,
+                                GWY_INTERPOLATION_BILINEAR);
+       }
 
     memcpy(tool->results.isel, isel, sizeof(isel));
     memcpy(tool->results.sel, sel, sizeof(sel));
@@ -709,6 +861,23 @@ update_label(GwySIValueFormat *units,
     gtk_label_set_markup(GTK_LABEL(label), buffer);
 }
 
+static void
+update_label_unc(GwySIValueFormat *units,
+             GtkWidget *label,
+             gdouble value, gdouble uncertainty)
+{
+    static gchar buffer[64];
+
+    g_return_if_fail(units);
+    g_return_if_fail(GTK_IS_LABEL(label));
+
+    g_snprintf(buffer, sizeof(buffer), "(%.*f±%.*f) %s%s",
+               units->precision, value/units->magnitude,
+               units->precision, uncertainty/units->magnitude,
+               *units->units ? " " : "", units->units);
+    gtk_label_set_markup(GTK_LABEL(label), buffer);
+}
+
 static void
 gwy_tool_stats_masking_changed(GtkWidget *button,
                                GwyToolStats *tool)
